

def get_occurances(haystack,needle,l=[],start=0,end=-1):
	""" return a list with all indices of needle in haystack """
	i = haystack.find(needle,start)
	if i == -1: return l
	return get_occurances(haystack,needle,l+[i],i+1)

def tuple_to_list(t):
	""" Convert a markup tuple:
			(text,[(color,pos),...])
		To a markup list:
			[(color,textpart),...]
		This is the opposite to urwid.util.decompose_tagmarkup
	"""
	(text,attrs) = t
	pc = 0
	l = []

	for (attr,pos) in attrs:
		if attr == None:
			l.append(text[pc:pc+pos])

		else:
			l.append((attr, text[pc:pc+pos]))
		pc += pos

	return l


def colorize_mult(text, needles, colors, base_color="default"):
	""" colorize more than one needle with a color.
		If base_color is given, the text not matching a needle
		is colored with the base_color.

		Example: colorize_mult("foo bar baz",["foo","baz"],
			{"foo":"green","baz":"blue"}) ->
			[("green","foo")," bar ",("blue","baz")]

				colorize_mult("foo bar baz",["foo","baz"],
			{"foo":"green","baz":"blue"}, base_color="red") ->
			[("green","foo"),("red"," bar "),("blue","baz")]
		Returns a markup list.
	"""
	# list sorted descending by needle length
	needles = sorted(needles, cmp=lambda a,b: (
		(len(a)>len(b) and -1) or (len(a)<len(b) and 1) or 0))

	def join(l,elem):
		nl = []
		for i in l[:-1]:
			if i != '':
				nl.append(i)
			nl.append(elem)
		if l and l[-1] != '':
			nl.append(l[-1])
		return nl

	l = [text]
	for needle in needles:
		nl = []
		for i in l:
			if isinstance(i,basestring):
				split = i.split(needle)
				if len(split) > 1:
					x = join(split,(colors[needle],needle))
					nl = nl + x
				else:
					nl.append(i)
			else:
				nl.append(i)
		l = nl

	if base_color != "default":
		nl = []
		for i in l:
			if isinstance(i,basestring):
				nl.append((base_color,i))
			else:
				nl.append(i)
		l = nl

	return l

def colorize(text, needle, color, base_color, start=0, end=-1):
	""" Color all occurances of needle in text in the given
		color. The markup will have the tuple-form like:
			(thetext,[(attr1,pos),...])
	"""
	occ = get_occurances(text,needle,start=start,end=end)
	p = 0
	l = []

	for i in occ:
		if i != p:
			if i-p > 0:
				l.append((base_color,i-p))
		l.append((color,len(needle)))
		p = i
	rest = len(text)-(p+len(needle))
	if rest > 0:
		l.append((base_color,rest))

	return (text, l)


if __name__ == "__main__":

	print "Testing tuple_to_list."

	r = tuple_to_list(("foobarbaz",[("red",3),("green",3),("blue",3)]))
	assert r == [("red","foo"),("green","bar"),("blue","baz")]

	r = tuple_to_list(("foobarbaz",[("red",3),(None,3),("blue",3)]))
	assert r == [("red","foo"),"bar",("blue","baz")]

	print "Success."

	print "Testing get_occurances."
	assert [0,1,2,3] == get_occurances("aaaa","a")
	assert [3] == get_occurances("abcde","d")
	assert [] == get_occurances("foo","x")
	print "Success."

	print "Testing colorize."
	text = "00:11 <nick> Das ist ein Test."
	color = "green"
	nick = "nick"
	base_color = "default"

	assert (text,[("default",7),("green",4),("default",19)]) == colorize(
		text,nick,color,base_color)
	print "Success."

	print "Testing colorize_mult."
	assert colorize_mult("foo bar baz",["foo","baz"],
			{"foo":"green","baz":"blue"}) == [
			("green","foo")," bar ",("blue","baz")]
	assert colorize_mult("foo bar baz",["foo","baz"],
			{"foo":"green","baz":"blue"}, base_color="red") == [
			("green","foo"),("red"," bar "),("blue","baz")]
	print "Success."



# coding: UTF-8
"""
Copyright (c) 2009 Marian Tietz
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
"""

import os
import sys

from xdg.BaseDirectory import xdg_config_home, xdg_data_home, xdg_cache_home
import ConfigParser

from typecheck import types
from messages import print_debug
from helper.escape import unescape_split, escape_join

prefix = ""
defaults = {}

config_parser = None
config_file = ""

def get_path(*c):
	return os.path.abspath(os.path.join(prefix, *c))

def set_defaults():
	"""
		Sets the default values.

		I know that the ConfigParser class has a
		method to set defaults but these defaults
		are global and not bound to a section.
		If you want a default nick color you can
		set it but if you have another option
		with the same name the default value will
		be the same and this sucks.

		A further point is that this way realizes
		"private" values which are not written
		to config files but can be used with the same
		API.
	"""
	global defaults

	defaults = {}
	defaults["aliase"] = {}

	defaults["nigiri"] = {}
	defaults["nigiri"]["wrap_input"] = "True"
	defaults["nigiri"]["command_char"] = "/"
	defaults["nigiri"]["shortcut_pattern"] = "meta [0-9]"
	defaults["nigiri"]["server_shortcuts"] = "true"
	defaults["nigiri"]["show_debug"] = "True"
	defaults["nigiri"]["logfile"] = os.path.join(xdg_cache_home, "sushi", "nigiri.txt")
	defaults["nigiri"]["locale_dir"] = get_path("..", "locale")
	defaults["nigiri"]["plugin_dirs"] = escape_join(",", (
			os.path.join(xdg_data_home, "nigiri", "plugins"),
			os.path.join(xdg_data_home, "chirashi"),
			get_path("plugins"),
			get_path("..", "chirashi")
		))

	defaults["chatting"] = {}
	defaults["chatting"]["quit_message"] = "Quit."
	defaults["chatting"]["part_message"] = "Part."
	defaults["chatting"]["last_log_lines"] = "15"
	defaults["chatting"]["nick_separator"] = ": "

	defaults["colors"] = {}
	defaults["colors"]["messages"] = "gray"
	defaults["colors"]["messages_own"] = "white"
	defaults["colors"]["messages_highlight"] = "red"
	defaults["colors"]["actions"] = "gray"
	defaults["colors"]["actions_own"] = "white"
	defaults["colors"]["informative"] = "blue"

	defaults["templates"] = {}
	# misc
	defaults["templates"]["datestring"] = "%H:%M"
	# messages
	defaults["templates"]["action"] = "%(time)s %(nick)s %(message)s"
	defaults["templates"]["action_own"] = "%(time)s %(nick)s %(message)s"
	defaults["templates"]["message"] = "%(time)s <%(prefix)s%(nick)s> %(message)s"
	defaults["templates"]["message_own"] = "%(time)s <%(prefix)s%(nick)s> %(message)s"
	defaults["templates"]["ctcp"] = "%(time)s -%(nick)s/%(target)s- %(message)s"
	defaults["templates"]["ctcp_own"] = "%(time)s -%(nick)s/%(target)s- %(message)s"
	defaults["templates"]["notice"] = "%(time)s *%(nick)s/%(target)s* %(message)s"
	defaults["templates"]["notice_own"] = "%(time)s *%(nick)s/%(target)s* %(message)s"

	# actions
	defaults["templates"]["invite"] = "%(time)s * %(nick)s invites %(who)s to %(channel)s"
	defaults["templates"]["invite_own"] = "%(time)s * You were invited by %(nick)s to %(channel)s."
	defaults["templates"]["join"] = "%(time)s * %(nick)s (%(host)s) has joined %(channel)s."
	defaults["templates"]["join_own"] = "%(time)s * You have joined %(channel)s."
	defaults["templates"]["kick"] = "%(time)s * %(who)s got kicked from %(channel)s by %(nick)s (%(reason)s)"
	defaults["templates"]["kick_own"] = "%(time)s * You got kicked from %(channel)s by %(nick)s (%(reason)s)"
	defaults["templates"]["nick"] = "%(time)s * %(nick)s is now known as %(new_nick)s."
	defaults["templates"]["nick_own"] = "%(time)s * You are now known as %(new_nick)s."
	defaults["templates"]["mode_list"] = "%(time)s * Modes for %(target)s: %(modes)s."
	defaults["templates"]["mode_list_own"] = "%(time)s * Modes set on you: %(modes)s."
	defaults["templates"]["mode"] = "%(time)s %(nick)s set %(mode)s%(param)s on %(target)s."
	defaults["templates"]["mode_own"] = "%(time)s You set %(mode)s%(param)s on %(target)s."
	defaults["templates"]["oper"] = "%(time)s * %(nick)s is now an OPER."
	defaults["templates"]["oper_own"] = "%(time)s * You are now an OPER."
	defaults["templates"]["part"] = "%(time)s * %(nick)s has left %(channel)s (%(reason)s)."
	defaults["templates"]["part_own"] = "%(time)s * You have left %(channel)s (%(reason)s)."
	defaults["templates"]["quit"] = "%(time)s * %(nick)s has quit (%(reason)s)."
	defaults["templates"]["quit_own"] = "%(time)s * You have quit (%(reason)s)."
	defaults["templates"]["topic"] = "%(time)s * %(nick)s has set the topic on %(channel)s to \"%(topic)s\"."
	defaults["templates"]["topic_own"] = "%(time)s * You have set the topic on %(channel)s to \"%(topic)s\"."
	defaults["templates"]["topic_anonymous"] = "* Topic of %(channel)s: %(topic)s"

	# informative
	defaults["templates"]["banlist_begin"] = "%(time)s Begin of banlist on channel %(channel)s."
	defaults["templates"]["banlist_item"] = ">> %(who)s %(mask)s %(when)s"
	defaults["templates"]["banlist_end"] = "%(time)s End of banlist. (%(channel)s)"
	defaults["templates"]["cannot_join"] = "%(time)s Can't join channel %(channel)s: %(reason)s"
	defaults["templates"]["list_begin"] = "%(time)s Begin of list."
	defaults["templates"]["list_item"] = ">> %(channel)s %(user)s %(topic)s"
	defaults["templates"]["list_end"] = "%(time)s End of list."
	defaults["templates"]["names_begin"] = "%(time)s Begin of names (%(channel)s)."
	defaults["templates"]["names_item"] = "[ %(row)s ]"
	defaults["templates"]["names_end"] = "%(time)s End of names."
	defaults["templates"]["no_such"] = "%(time)s No such %(type)s: %(target)s."
	defaults["templates"]["whois_begin"] = "%(time)s Begin of whois (%(target)s)."
	defaults["templates"]["whois_item"] = ">> %(nick)s: %(message)s"
	defaults["templates"]["whois_end"] = "%(time)s End of whois."
	defaults["templates"]["dcc_new_incoming"] = "%(time)s Incoming file transfer (ID:%(id)d) from %(sender)s: \"%(filename)s\" (%(size)d)."
	defaults["templates"]["dcc_file_auto_accept"] = "%(time)s Auto accepted file transfer from %(sender)s: \"%(filename)s\" (%(size)d)."

	# Add default sections to config parser
	# so setting is easier
	for section in defaults.keys():
		try:
			config_parser.add_section(section)
		except ConfigParser.DuplicateSectionError:
			continue

	# sections defined below are not added to the configParser and
	# can't be set by the set method (will raise NoSectionError)
	pass

@types (name=str, address=str, port=int, user=str, password=str, log=bool)
def read_config_file():
	"""
	Reads the config file.
	"""
	success = config_parser.read([config_file])

	if not success:
		print "Failed to parse config file '%s'" % config_file
		return False

	return True

def write_config_file():
	"""
	Writes the config values from the
	ConfigParser object into the given file (config_file)
	"""
	if not config_parser:
		print "Config module not loaded. I don't save anything."
		return

	f = file(config_file, "w")
	config_parser.write(f)
	f.close()

@types (section=basestring)
def create_section(section):
	"""
		creates config section `section`.
	"""
	if not config_parser or config_parser.has_section(section):
		return False
	config_parser.add_section(section)
	return True

@types (section=basestring)
def remove_section(section):
	"""
		removes the section
	"""
	if not config_parser or not config_parser.has_section(section):
		return False

	config_parser.remove_section(section)
	return True

@types (section=basestring, option=basestring)
#@on_fail (print_debug, "ConfigError while setting %s:%s to %s")
def set(section, option, value):
	"""
		Sets in the section the option to value.
		On success the method returns True.
	"""
	if not config_parser:
		return False

	try:
		config_parser.set(section, option, str(value))
	except ConfigParser.NoSectionError:
		return False
	else:
		return True

@types (section=basestring, option=basestring, l=list)
def set_list(section, option, l):
	"""
	join the list l to a string separated
	by , and set it as value to option.
	Return False on error, else True.
	"""
	s = escape_join(",", l)

	if None == s:
		return False

	return set(section, option, s)

@types (section=basestring, option=basestring, value=basestring)
def append_list(section, option, value):
	"""
	add value to the list identified by option
	"""
	v = get_list(section, option, [])
	v.append(value)
	return set_list(section, option, v)

@types (section=basestring, option=basestring)
def unset(section, option):
	"""
		Removes the option in the section.
		Returns True on success otherwise False.
	"""
	if not config_parser:
		return False
	try:
		config_parser.remove_option(section, option)
	except ConfigParser.NoSectionError,e:
		print "Unsetting ('%s','%s') failed: %s" % (section,option,e)
		return False
	return True

@types (section=basestring, option=basestring)
def get(section, option="", default=None):
	"""
		Returns the value for option in section, on
		error the method returns default.

		If option is not given, the whole section
		is returned as dictionary of type {option:value}.
		If there are default values for the section they
		will be merged in.

		The `option`-value is handled case-insensitive.

		get("tekka") will return {"server_shortcuts":"1"}
		get("tekka","server_shortcuts") will return "1"
		get("tekki") will return default
	"""

	if not config_parser:
		return default

	if not option:
		# get the whole section

		if config_parser.has_section(section):
			# the section is noticed by the parser,
			# merge with defaults (if any) and return
			# the value-dict

			new = dict(config_parser.items(section))

			if defaults.has_key(section):
				# merge defaults + config values together
				copy = defaults[section].copy()
				copy.update(new)
				return copy
			else:
				return new

		elif defaults.has_key(section):
			# the config parser does not know the
			# section but it's found in defaults-dict.
			# This is probably a private section.
			return defaults[section]

		else:
			return default

	else:
		# get specific option

		try:
			return config_parser.get(section, option)
		except (ConfigParser.NoOptionError, ConfigParser.NoSectionError),e:
			if defaults.has_key(section):
				try:
					return defaults[section][option]
				except KeyError:
					return default
		else:
			return default

	# usually this is not reached
	return default

@types (section=basestring, option=basestring)
def get_list(section, option, default):
	"""
		Splits the option in the section for ","
		and returns a list if the splitting was
		successful. Else the method will return
		"default".
	"""
	res = get(section, option, default)

	if res == default:
		return default

	l = unescape_split(",", res)

	if not l:
		return default
	return list(l)

@types (section=basestring, option=basestring)
def get_bool(section, option, default=False):
	"""
		Returns True or False if the value is
		set or unset.
	"""
	res = get(section, option, default)

	if res == default:
		return default

	if res.lower() == "true" or res == "1":
		return True

	return default

@types (section=basestring, option=basestring)
def get_default(section, option=""):
	"""
	Returns the default value for the option
	in the given section. If no option is given
	(option = None) all defaults of the given
	section are returned as a dictionary.
	If there are no defaults, None is returned.
	"""
	if not option:
		if defaults.has_key(section):
			return defaults[section]
	else:
		if defaults.has_key(section):
			if defaults[section].has_key(option):
				return defaults[section][option]
	return None

@types (path=basestring)
def check_config_file(path):
	""" check if config file exists and create it if not """
	if not os.path.exists (path):
		# create the directories
		try:
			os.makedirs (os.path.join (os.path.split (path)[0]))
		except os.error:
			print "Error while creating neccessary directories: %s"\
				% (os.error)
			return False

		try:
			f = file (path, "w")
		except BaseException,e:
			print "Error while creating config file: %s" % (e)
			return False
		else:
			f.close()

		return True
	else:
		return True
	return False

def setup():
	"""
	Find the usual location of the config dir
	(XDG_CONFIG_HOME/sushi) and parse the
	config file (nigiri) if found.
	"""
	global config_parser, config_file
	global prefix

	if os.path.islink(sys.argv[0]):
		link = os.readlink(sys.argv[0])

		if not os.path.isabs(link):
			link = os.path.join(os.path.dirname(sys.argv[0]), link)

		prefix = os.path.dirname(os.path.abspath(link))
	else:
		prefix = os.path.dirname(os.path.abspath(sys.argv[0]))

	config_parser = ConfigParser.ConfigParser()
	set_defaults()

	config_file = os.path.join (xdg_config_home, "sushi", "nigiri")

	if not check_config_file(config_file):
		print "Config file creation failed. Aborting."
		return

	read_config_file()




# coding: UTF-8
"""
Copyright (c) 2009 Marian Tietz
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
"""

# UHF = ultra high frequency :]

import gtk
import logging
from gobject import idle_add

from .. import gui
from .. import config
from .. import helper
from ..lib.expanding_list import ExpandingList

widgets = None

MESSAGE_TYPES=gui.tabs.MSGTYPES

def go_valid_types_toggled(typename, checkbtn):
	""" add the typename to the list of valid message types if
		the checkbox is checked, otherwise delete it
	"""
	section = "general_output"
	option = "valid_types"

	l = config.get_list(section, option, [])

	if typename not in l:
		if checkbtn.get_active():
			config.append_list(section, option, typename)

	else:
		if not checkbtn.get_active():
			del l[l.index(typename)]
			config.set_list(section, option, l)


def generalOutputFilterList_instanced_widget_cb(elist, row, column, obj):
	if column == 0:
		model = gtk.ListStore(str)
		obj.set_model(model)

		renderer = gtk.CellRendererText()
		obj.pack_start(renderer, True)
		obj.add_attribute(renderer, "text", 0)

		for row in MESSAGE_TYPES:
			model.append((row,))

def fillTekka():
	table = widgets.get_object("tekkaTable")

	# set checkbuttons
	for child in table.get_children():

		if type(child) != gtk.CheckButton:
			continue

		name = gtk.Buildable.get_name(child)
		bval = config.get_bool("tekka", name)

		child.set_active(bval)

        # line spacing
        spacing = int(config.get("tekka", "line_spacing"))
        widgets.get_object("line_spacing").set_value(spacing)

	# set font labels
	font = config.get("tekka", "font")
	widgets.get_object("fontSelectButton").set_font_name(font)


def fillColors():
	for key in ("own_nick", "own_text", "notification",
				"text_message", "text_action", "nick",
				"text_highlightmessage", "text_highlightaction",
				"last_log", "rules_color"):

		color = helper.color.get_color_by_key(key)

		widgets.get_object(key).set_color(color)

	btn = widgets.get_object("rules_color_yesno")
	btn.set_active(config.get_bool("tekka","text_rules"))
	btn.toggled()

	btn = widgets.get_object("auto_rule_color")
	btn.set_active(config.get("colors","rules_color") == "auto")
	btn.toggled()

	btn = widgets.get_object("irc_colors")
	btn.set_active(config.get_bool("colors","irc_colors"))
	btn.toggled()



def fillChatting():
	highlightList = widgets.get_object("highlightList")

	for key in ("quit_message", "part_message", "time_format"):
		val = config.get("chatting", key)
		widgets.get_object(key).set_text(val)

	i = 0
	for highlight in config.get_list("chatting", "highlight_words", []):
		highlightList.get_widget_matrix()[i][0].set_text(highlight)
		highlightList.add_row()
		i+=1

	highlightList.remove_row(i)

	val = config.get("chatting", "last_log_lines", default=0)
	widgets.get_object("last_log_lines").set_value(float(val))

def fillNickColors():
	nickColorsList = widgets.get_object("nickColorsList")
	widgets.get_object("nick_contrast_colors").set_active(
		config.get_bool("colors", "nick_contrast_colors"))

	colors = config.get_list("colors", "nick_colors", [])

	if not colors:
		return

	i = 0

	for color in colors:
		try:
			c = gtk.gdk.Color(color)
		except:
			c = gtk.gdk.Color()

		nickColorsList.get_widget_matrix()[i][0].set_color(c)
		nickColorsList.add_row()
		i+=1

	nickColorsList.remove_row(i)

def fillGeneralOutputFilters():
	types = config.get_list("general_output", "valid_types", [])

	table = {}
	table[gui.tabs.MESSAGE] = "type_message"
	table[gui.tabs.ACTION] = "type_action"
	table[gui.tabs.HIGHMESSAGE] = "type_highlightmessage"
	table[gui.tabs.HIGHACTION] = "type_highlightaction"

	for type in types:
		if not table.has_key(type):
			logging.error("Invalid key: %s" % (type,))
			continue
		w = widgets.get_object(table[type])
		w.set_active(True)

	# (type, server, channel), (type, server), ...
	filter = config.get_list("general_output", "filter", [])

	generalOutputFilterList = widgets.get_object("generalOutputFilterList")

	i=0
	for tuple in filter:
		try:
			e_tuple = eval(tuple)
		except BaseException as e:
			logging.error("Tuple '%s' in filter rule is malformed: %s" % (
				tuple, e))
			continue

		widget_row = generalOutputFilterList.get_widget_matrix()[i]
		combobox = widget_row[0]

		try:
			type_index = MESSAGE_TYPES.index(e_tuple[0])
		except ValueError:
			logging.error("Unknown message type '%s'." % (e_tuple[0]))
			continue
		else:
			combobox.set_active(type_index)

		if len(e_tuple) >= 2:
			widget_row[1].set_text(e_tuple[1])

		if len(e_tuple) == 3:
			widget_row[2].set_text(e_tuple[2])

		generalOutputFilterList.add_row()
		i+=1
	generalOutputFilterList.remove_row(i)


def applyNickColors():
	nickColorsList = widgets.get_object("nickColorsList")
	config.set_list("colors","nick_colors",
		[n[0].get_color().to_string()
		for n in nickColorsList.get_widget_matrix() if n and len(n) >= 1])

def applyChatting():
	highlightList = widgets.get_object("highlightList")
	config.set_list("chatting", "highlight_words", [n[0].get_text() for n in highlightList.get_widget_matrix() if n and n[0].get_text()])

def applyGeneralOutputFilter():
	generalOutputFilterList = widgets.get_object("generalOutputFilterList")
	filter_list = []
	header = ("type", "server", "channel")

	for widget_row in generalOutputFilterList.get_widget_matrix():
		cbox = widget_row[0]

		if not cbox.get_model() or cbox.get_active() == -1:
			logging.error("No message type selected.")
			continue

		mtype = cbox.get_model()[cbox.get_active()][0]

		server = widget_row[1].get_text()
		channel = widget_row[2].get_text()

		f_tuple = (str(mtype), str(server), str(channel))

		filter_list.append(str(f_tuple))

	config.set_list("general_output", "filter", filter_list)

""" tekka page signals """

def tekka_show_status_icon_toggled(button):
	config.set("tekka", "show_status_icon",
			str(button.get_active()))
	gui.mgmt.visibility.show_status_icon(button.get_active())

def tekka_hide_on_close_toggled(button):
	config.set("tekka", "hide_on_close",
			str(button.get_active()))

def tekka_font_clicked(button):
	font = button.get_font_name()

	if font:
		config.set("tekka", "font", font)
		gui.mgmt.apply_new_font()

def tekka_use_default_font_toggled(button):
	config.set("tekka", "use_default_font",
			str(button.get_active()))

	gui.mgmt.apply_new_font()

def tekka_line_spacing_changed(spinner):
	value = int(spinner.get_value())
	config.set("tekka", "line_spacing", str(value))

        for tab in gui.tabs.get_all_tabs():
            tab.window.textview.set_line_spacing(value)

def tekka_auto_expand_toggled(button):
	config.set("tekka", "auto_expand",
			str(button.get_active()))

def tekka_rgba_toggled(button):
	config.set("tekka", "rgba",
			str(button.get_active()))

def tekka_close_maki_on_close_toggled(button):
	config.set("tekka", "close_maki_on_close",
			str(button.get_active()))

""" colors page signals """

def colors_color_button_clicked(button):
	color_name = gtk.Buildable.get_name(button)

	def open_contrast_dialog(color_name):

		def response_cb(dialog, id, dialog_wrap):
			if id == gtk.RESPONSE_OK:
				(rcolor, ccolor) = dialog_wrap.get_current_color()

				if rcolor and not ccolor:
					value = str(rcolor)
					button.set_color(rcolor)
				elif ccolor:
					value = str(ccolor)
					button.set_color(rcolor)
				else:
					value = None

				if value:
					config.set("colors", color_name, value)
			dialog.destroy()


		dialog = gui.dialogs.show_dialog("contrast")

		conf_color = config.get("colors", color_name)

		if helper.color.is_contrast_color(conf_color):
			dialog.set_current_contrast_color(int(conf_color))
		else:
			dialog.set_current_rgb_color(button.get_color())

		dialog.connect("response", response_cb, dialog)

	open_contrast_dialog(color_name)


def colors_rules_color_written(button):
	colors_set_color_from_button(button, "rules_color")

def colors_rules_autodetect_toggled(button):
	if button.get_active():
		config.set("colors","rules_color","auto")
	widgets.get_object("rules_color").set_sensitive(not button.get_active())
	widgets.get_object("reset_rules_color").set_sensitive(
		not button.get_active())

def colors_rules_color_yesno_toggled(button):
	flag = button.get_active()
	config.set("tekka", "text_rules", str(flag))
	widgets.get_object("auto_rule_color").set_sensitive(flag)
	widgets.get_object("rules_color").set_sensitive(flag)
	widgets.get_object("reset_rules_color").set_sensitive(flag)

def colors_irc_colors_toggled(button):
	config.set("colors","irc_colors", str(button.get_active()))

def reset_color(color_key):
	""" reset the color to it's default value (contrast color index) """
	if config.is_default("colors",color_key):
		return
	config.reset_value("colors",color_key)
	widgets.get_object(color_key).set_color(
		helper.color.get_color_by_key(color_key))

""" chatting page signals """

def chatting_quit_message_written(entry, event):
	text = entry.get_text()
	config.set("chatting", "quit_message", text)

def chatting_part_message_written(entry, event):
	text = entry.get_text()
	config.set("chatting", "part_message", text)

def chatting_time_format_written(entry, *x):
	text = entry.get_text()
	config.set("chatting", "time_format", text)

def chatting_log_lines_changed(button):
	value = int(button.get_value())

	if value < 0:
		return

	config.set("chatting", "last_log_lines", str(value))

""" nick colors page signals """

def nick_contrast_colors_toggled_cb(button):
	config.set("colors", "nick_contrast_colors", str(button.get_active()))

	ncl = widgets.get_object("nickColorsList")

	if ncl:
		ncl.set_sensitive(not button.get_active())

""" advanced page signals """

def advanced_advancedSettingsClicked(button):
	d = gui.dialogs.loadDialog("advancedPreferences")
	if not d:
		logging.error("advanced settings dialog setup failed")
		return
	d.run()

""" setup/run/maintenace methods """

def setup():
	"""
	read ui stuff
	"""
	global widgets

	widgets = gui.builder.load_dialog("preferences")

	sigdic = {
	# tekka page
		"tekka_show_status_icon_toggled": tekka_show_status_icon_toggled,
		"tekka_hide_on_close_toggled": tekka_hide_on_close_toggled,
		"tekka_font_clicked": tekka_font_clicked,
		"tekka_use_default_font_toggled": tekka_use_default_font_toggled,
                "tekka_line_spacing_changed": tekka_line_spacing_changed,
		"tekka_auto_expand_toggled": tekka_auto_expand_toggled,
		"tekka_rgba_toggled": tekka_rgba_toggled,
		"tekka_close_maki_on_close_toggled": tekka_close_maki_on_close_toggled,
	# colors page
		"colors_color_button_clicked": colors_color_button_clicked,
		"colors_rules_autodetect_toggled": colors_rules_autodetect_toggled,
		"colors_rules_color_yesno_toggled": colors_rules_color_yesno_toggled,
		"colors_irc_colors_toggled": colors_irc_colors_toggled,

	# chatting page
		"chatting_quit_message_written": chatting_quit_message_written,
		"chatting_part_message_written": chatting_part_message_written,
		"chatting_time_format_written": chatting_time_format_written,
		"chatting_log_lines_changed": chatting_log_lines_changed,
	# general output page
		"go_type_message_toggled":
			lambda *x: go_valid_types_toggled(gui.tabs.MESSAGE,*x),
		"go_type_action_toggled":
			lambda *x: go_valid_types_toggled(gui.tabs.ACTION,*x),
		"go_type_highlightmessage_toggled":
			lambda *x: go_valid_types_toggled(gui.tabs.HIGHMESSAGE,*x),
		"go_type_highlightaction_toggled":
			lambda *x: go_valid_types_toggled(gui.tabs.HIGHACTION,*x),
		"generalOutputFilterList_instanced_widget_cb":
			generalOutputFilterList_instanced_widget_cb,
	# nick colors page
		"nick_contrast_colors_toggled_cb": nick_contrast_colors_toggled_cb,
	# advanced page
		"advanced_advancedSettingsClicked": advanced_advancedSettingsClicked
	}

	def cb_factory(key):
		def cb(w):
			return reset_color(key)
		return cb

	# add color reset handler
	for key in ("own_nick", "own_text", "notification",
				"text_message", "text_action", "nick",
				"text_highlightmessage", "text_highlightaction",
				"last_log", "rules_color"):

		sigdic["reset_"+key+"_clicked"] = cb_factory(key)

	widgets.connect_signals(sigdic)


def dialog_response_cb(dialog, response_id):
	applyNickColors()
	applyGeneralOutputFilter()
	applyChatting()

	dialog.destroy()

def run():
	dialog = widgets.get_object("preferencesDialog")

	# the widget is not initialized with a first row
	# (no_first_row in ui file set), do it here.
	widgets.get_object("generalOutputFilterList")._add_row(0)

	fillTekka()
	fillColors()
	fillChatting()
	fillNickColors()
	fillGeneralOutputFilters()

	main_window = gui.mgmt.widgets.get_object("main_window")
	dialog.set_transient_for(main_window)

	dialog.connect("response", dialog_response_cb)
	dialog.show_all()






from tekka.lib.spell_entry import SpellEntry
from tekka.lib.search_toolbar import SearchBar
from tekka.lib.output_shell import OutputShell
from tekka.lib.output_window import OutputWindow
from tekka.lib.htmlbuffer import HTMLBuffer
from tekka.lib.general_output_buffer import GOHTMLBuffer
from tekka.lib.expanding_list import ExpandingList
from tekka.lib.contrast_color_table import ContrastColorTable
from tekka.lib.custom_color_button import CustomColorButton

print "TEKKA WIDGETS LOADED."



# coding: UTF-8
"""
Copyright (c) 2009 Marian Tietz
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
"""

"""
- no entries, no scrolling
- typed but not confirmed stuff is the latest item
- confirmed stuff is pushed into a stack

- button up: no entries: abort
- button up: an entry: set position to entry, return entry
- button up: on top: abort

- button down: check for typed OR no entries: abort
- button down: an entry: set position to entry, return entry
- button down: on bottom: check for typed OR abort
"""

from ..typecheck import types

class InputHistory (object):

	_history = []
	_size = 20
	_position = None
	_get_text_callback = lambda: ""
	_origin_text = ""

	@types (size=int)
	def __init__ (self, size=20, text_callback=lambda: ""):
		self._size = size
		self._history = []
		self._position = None
		self._origin_text = ""
		self._get_text_callback = text_callback

	def reset(self):
		self._position = None

	@types (fun=type(lambda:""))
	def set_text_callback(self, fun):
		self._get_text_callback = fun

	@types (entry=basestring)
	def add_entry (self, entry):
		""" add a string to the history """
		self._history.append (entry)

		if len(self._history) == self._size:
			del self._history[0]

	def get_previous(self):
		""" walk the stack down to zero, button up """
		if self._position == None:
			# first call, return the latest item
			if not self._history:
				return None
			# get the original text
			self._origin_text = self._get_text_callback()
			self._position = len(self._history)-1
			return self._history[-1]

		if self._position == 0:
			# abort
			return None

		self._position -= 1
		return self._history[self._position]

	def get_next(self):
		""" walk the stack up to self._size, button down """
		if self._position == None:
			# first call, get the origin text and return None
			self._origin_text = self._get_text_callback()
			return None

		if self._position == len(self._history)-1:
			# the old input should be restored by
			# the calling method
			self._position = None
			return self._origin_text

		self._position += 1
		return self._history[self._position]

	@types (index=int)
	def get_value(self, index):
		try:
			return self._history[index]
		except IndexError:
			return None
		return None



# coding: UTF-8
"""
Copyright (c) 2008 Marian Tietz
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
"""
from gettext import gettext as _

from types import MethodType, FunctionType

from . import gui
from . import com

from .helper import color
from .com import sushi
from .typecheck import types

_commands = {}

@types(text=basestring)
def parseInput(text):
	"""
	1. Parse color codes (%Cn[,m]),
	2. Split text for blank, strip the command
	   and search for it in _commands-dict.
	3. Call the underlying function if found.

	   Paramters for the function are:
	   - the current server or None
	   - the current channel or None
	   - A list of words typed after the command
	     ("parameters") split by space
	"""
	if not text:
		return

	# parse %C tags
	text = color.parse_color_markups_to_codes(text)

	serverTab,channelTab = gui.tabs.get_current_tabs()

	if ((channelTab and not channelTab.connected)
		or (serverTab and not serverTab.connected)):
		# there is no connection in this tab so
		# if you're typing something, it would have
		# no effect. So warn the user.
		warnNoConnection(serverTab)

	if text[0] != "/" or text[:2] == "//":
		# this is no command

		if not channelTab:
			# no command AND no channel is nonsense.
			# normal text is useless in context
			# with server tabs
			return

		# strip first slash if it's a fake command
		if text[0] == "/":
			text = text[1:]

		if channelTab.is_channel() and not channelTab.joined:
			warnNotJoined(channelTab)

		com.sendMessage(serverTab.name, channelTab.name, text)

	else:
		# we got a command here

		argv = text[1:].rstrip().split(" ")
		cmd = argv[0]

		if not cmd:
			# / typed
			return gui.mgmt.myPrint("No command given.")

		# search for the command
		global _commands

		if not _commands.has_key(cmd):
			# command not found, look if we
			# can send it as RAW.

			if not serverTab:
				return gui.mgmt.myPrint("No server active.")

			# build raw command
			raw = cmd.upper()

			if len(argv) > 1:
				raw +=  " " + " ".join(argv[1:])

			gui.mgmt.myPrint(_(
				u"• Unknown command “%(command)s”, "\
				u"sending raw command “%(raw)s”.") % {
					"command": cmd,
					"raw": raw })
			sushi.raw(serverTab.name, raw)

		else:
			_commands[cmd](serverTab, channelTab, argv[1:])


@types(command=basestring,function=(MethodType,FunctionType))
def add_command(command, function):
	"""
		Add a command.
		Returns True on success, otherwise False.
	"""
	global _commands

	if _commands.has_key(command):
		return False

	_commands[command] = function

	return True


@types(command=basestring)
def remove_command(command):
	"""
		Removes a command.
		Returns True on success, otherwise False.
	"""
	global _commands

	if _commands.has_key(command):
		del _commands[command]
		return True

	return False




# coding: UTF-8
"""
Copyright (c) 2009 Marian Tietz
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
"""


"""
provide methods for warnings, notifications, debug and errors
"""

import sys
import urwid
import urwid.util
import time

import config
from typecheck import types
import tabs
import helper

main_window = None

def setup (mw):
	global main_window
	main_window = mw

class FormattedMessage(object):

	@property
	def markup_cb(self): return self._markup_cb
	@markup_cb.setter
	def markup_cb(self, cb): self._markup_cb = cb

	@types( category = basestring, template = basestring,
			values = dict, highlight = bool, own = bool )
	def __init__(self, category, template, values, base_color,
				 highlight = False, own = False):
		self.category = category
		self.template = template
		self.values = values
		self.highlight = highlight
		self.base_color = base_color
		self.own = own 	# we triggered that/we are meant by it

		self.markup_cb = None
		self.markup_cb_kwargs = {}

	def __str__(self):
		try:
			return self.template % (self.values)
		except KeyError,e:
			return "TEMPLATE_MISSING_KEY(%s)" % (e)

	def _markup(self):
		return [(self.base_color, unicode(self))]

	def markup(self):
		if self.markup_cb:
			return self.markup_cb(self, **self.markup_cb_kwargs)
		return self._markup()


@types (mtype=basestring, template_id=basestring, values=dict)
def format_message(mtype, template_id, values, highlight=False, own=False):
	""" factory method for FormattedMessage """
	if highlight:
		generic_type = mtype + "_highlight"
	elif own:
		generic_type = mtype + "_own"
		template_id = template_id + "_own"
	else:
		generic_type = mtype

	values["time"] = time.strftime(config.get("templates", "datestring"))

	base_color = config.get("colors", generic_type, "default")
	template = config.get("templates", template_id)

	msg = FormattedMessage(
		mtype, template, values, base_color, highlight, own)

	if template == None:
		msg.template = "TEMPLATE_ERROR(%s)" % template_id

	return msg

@types(msg = (basestring, list, FormattedMessage))
def print_tab(dest_tab, msg, msgtype="informative"):
	if not main_window:
		raise ValueError, "No main_window found."

	tablist = tabs.tree_to_list(main_window.servers)

	try:
		i = tablist.index(dest_tab)

	except ValueError:
		print_error("print_tab to invalid destinaton '%s'." % dest_tab)
		return

	else:
		if isinstance(msg, FormattedMessage):
			markup = msg.markup()
			msgtype = msg.category
			if isinstance(markup,tuple):
				# ("...",[(color,pos),...])
				new_markup = helper.markup.tuple_to_list(markup)
				textItem = urwid.Text(new_markup)
			elif isinstance(markup,list):
				# [(color,text),...]
				textItem = urwid.Text(markup)
			else:
				textItem = urwid.Text(markup)
		else:
			textItem = urwid.Text(msg)

		tablist[i].output_walker.append(textItem)

		if main_window.current_tab != dest_tab:
			dest_tab.add_status(msgtype)
			main_window.update_divider()
		else:
			main_window.body.scroll_to_bottom()

def print_tab_notification(tab, msg):
	print_tab(tab, "*** Notification: " + msg)

def print_tab_error(tab, msg):
	print_tab(tab, "!!! Error: " + msg)

def print_normal(msg, *args, **dargs):
	main_window.print_text(msg)

def print_error (msg, *args, **dargs):
	main_window.print_text("!!! Error: " + msg)

def print_notification (msg):
	main_window.print_text("*** Notification: " + msg)

def print_debug (msg, *args, **dargs):
	if not config.get_bool("nigiri", "show_debug"):
		return
	main_window.print_text("=== Debug: " + msg)



# coding: UTF-8
"""
Copyright (c) 2009 Marian Tietz
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
"""

import gtk
import glib
import gobject
import dbus
import logging
from gettext import gettext as _
import traceback

from .. import config
from ..com import parse_from, sushi
from .. import gui
from ..helper.dcc import s_incoming

widgets = None

(COL_STATUS,
 COL_ID,
 COL_SERVER,
 COL_PARTNER,
 COL_FILE,
 COL_SIZE,
 COL_PROGRESS,
 COL_SPEED) = range(8)


class DCCWatcher(object):

	def __init__(self):
		self.timer_id = glib.timeout_add(1000, self._refresh_sends)

		self._init_cache()

	def stop(self):
		""" stop watching for dcc sends periodically """
		gobject.source_remove(self.timer_id)

	def refresh(self):
		""" manually refresh the list by calling this method """
		self._refresh_sends()

	def _init_cache(self):
		self.last_sends = []
		self.row_id_map = {}

	def _refresh_sends(self):
		sends = sushi.dcc_sends()

		if (sends == None and not sushi.connected) or len(sends) == 0:
			return

		view = widgets.get_object("transferView")
		store = view.get_model()

		# all ids
		act_sends = sends[0]

		to_remove = set(self.last_sends) - set(act_sends)
		to_update = set(act_sends) - to_remove

		for i in range(len(sends[0])):
			id, server, sender, filename, size, progress, speed, status = \
			[sends[n][i] for n in range(len(sends))]

			if id in to_update:
				if self.row_id_map.has_key(id):
					# update existing entry
					iter = self.row_id_map[id].iter

					store.set(iter,
						COL_STATUS, status,
						COL_SIZE, size,
						COL_PROGRESS, get_progress(progress, size),
						COL_SPEED, speed)
				else:
					# add new entry

					if not config.get_bool("dcc", "show_ident_in_dialog"):
						sender = parse_from(sender)[0]

					iter = store.append(row = (
						status, id, server,
						sender, filename,
						size, get_progress(progress, size),
						speed))

					self.row_id_map[id] = store[store.get_path(iter)]

		for id in to_remove:
			if self.row_id_map.has_key(id):
				store.remove(self.row_id_map[id].iter)

		self.last_sends = act_sends

		return True


def get_progress(p, s):
	return int(float(p)/s*100)

def cancel_transfer(transferID, watcher):
	sushi.dcc_send_remove(transferID)
	watcher.refresh()

def get_selected_transfer_id():
	view = widgets.get_object("transferView")
	store = view.get_model()
	cursor = view.get_cursor()

	try:
		id = dbus.UInt64(store[cursor[0]][COL_ID])
	except:
		return None
	else:
		return id

def dialog_response_cb(dialog, id, watcher):
	if id == 333: # FIXME:  replace this with a meaningful ID
				  # FIXME:: or connect to the button directly
		# remove was clicked
		def ask_are_you_sure():
			# ask if the user is sure about removing the transfer

			def dialog_reponse_cb(dialog, id, transferID):
				if id == gtk.RESPONSE_YES:
					# yes, remove it!
					cancel_transfer(transferID, watcher)
				dialog.destroy()

			transferID = get_selected_transfer_id()

			if None == transferID:
				gui.mgmt.show_error_dialog(
					title=_("No transfer selected!"),
					message=_("You must select a transfer to remove it."))

			else:
				d = gui.builder.question_dialog(
					title = _("Remove file transfer?"),
					message = _("Are you sure you want to remove the "
						"file transfer %(id)d?" % {
							"id": transferID }))
				d.connect("response", dialog_reponse_cb, transferID)
				d.show()

		ask_are_you_sure()

	else:
		global widgets

		watcher.stop()
		dialog.destroy()

		widgets = None

def run():
	dialog = widgets.get_object("DCCDialog")

	if dialog.get_property("visible"):
		return

	watcher = DCCWatcher()

	main_window = gui.mgmt.widgets.get_object("main_window")
	dialog.set_transient_for(main_window)

	dialog.connect("response", dialog_response_cb, watcher)
	dialog.show()

def setup():
	global widgets

	if widgets != None:
		return

	widgets = gui.builder.load_dialog("dcc")

	# add direction icon column
	def type_symbol_render_cb(column, renderer, model, iter):
		status = model.get(iter, COL_STATUS)
		if status:
			if status[0] & s_incoming:
				# incoming
				renderer.set_property("stock-id", gtk.STOCK_GO_DOWN)
			else:
				# outgoing
				renderer.set_property("stock-id", gtk.STOCK_GO_UP)

	widgets.get_object("statusColumn").set_cell_data_func(
			widgets.get_object("statusRenderer"),
			type_symbol_render_cb)




import gobject
import time

from ... import config

from ...helper import color
from ...typecheck import types

from . import tab
from . import util

from .messages import *



class TekkaQuery(tab.TekkaTab):
	""" Class for typical query-tabs """

	def __init__(self, name, server, textview=None):
		super(TekkaQuery,self).__init__(name, textview)

		self.server = server

	def is_query(self):
		return True

	def markup(self):
		italic = False
		bold = False
		foreground = None

		base = self.name

		if not self.connected:
			base = "<span strikethrough='true'>"+base+"</span>"

		if ACTION in self.newMessage:
			italic = True

		if MESSAGE in self.newMessage:
			bold = True

		if (HIGHMESSAGE in self.newMessage
			and HIGHACTION in self.newMessage):
			foreground = util._markup_color("new_highlightmessage")
		elif HIGHMESSAGE in self.newMessage:
			foreground = util._markup_color("new_highlightmessage")
			bold = True
		elif HIGHACTION in self.newMessage:
			foreground = util._markup_color("new_highlightaction")

		markup = "<span "
		if italic:
			markup += "style='italic' "

		if bold:
			markup += "weight='bold' "

		if foreground:
			markup += "foreground='%s'" % foreground

		markup += ">%s</span>" % base

		return markup


	def write(self, timestamp, message, msgtype=MESSAGE,
	no_general_output = False, **kwargs):
		""" write [<timestamp>] <message> to this tab's buffer """

		timestring = time.strftime(
					config.get("chatting", "time_format", "%H:%M"),
					time.localtime(timestamp))

		cString = color.colorize_message(msgtype, message)

		outputString = "[%s] %s" % (timestring, cString)

		buffer = self.window.textview.get_buffer()
		buffer.insert_html(buffer.get_end_iter(), outputString, **kwargs)

		if not self.is_active():

			if (config.get_bool("tekka", "show_general_output")
			and not no_general_output):

				# write it to the general output, also
				util._write_to_general_output(
					msgtype,
					timestring,
					self,
					message)

		def notify():
			self.set_new_message(msgtype)
			return False

		gobject.idle_add(notify)





import command_handler as cmdh

""" This file is more likely to find programming errors than
	to check logical flaws. It will NOT show you if a command
	acted correctly, it will only show if it's not faulty
	programmed.
"""

def start_test(gui):
	_commands = {
		"connect" : cmdh.cmd_connect,
		"nick" : cmdh.cmd_nick,
		"part" : cmdh.cmd_part,
		"join" : cmdh.cmd_join,
			"j" : cmdh.cmd_join,
		"me"   : cmdh.cmd_action,
		"kick" : cmdh.cmd_kick,
		"mode" : cmdh.cmd_mode,
		"topic": cmdh.cmd_topic,
		"quit" : cmdh.cmd_quit,
		"away" : cmdh.cmd_away,
		"back" : cmdh.cmd_back,
	"nickserv" : cmdh.cmd_nickserv,
		"ctcp" : cmdh.cmd_ctcp,
		"names" :  cmdh.cmd_names,
		"notice" : cmdh.cmd_notice,
		"msg" : cmdh.cmd_message,
		"oper" : cmdh.cmd_oper,
		"list" : cmdh.cmd_list,
		"raw" : cmdh.cmd_raw,
		"stoplist" : cmdh.cmd_stop_list,
		"whois" : cmdh.cmd_whois,
		"query":  cmdh.cmd_query,
		"clear":  cmdh.cmd_clear,
		"help":  cmdh.cmd_help
	}

	for (cmd,hdl) in _commands.items():
		gui.mgmt.myPrint("Testing %s with None args..." % (cmd))
		hdl(None, None, [])

	gui.mgmt.myPrint("Testing simple commands...")
	s,c = gui.tabs.get_current_tabs()

	gui.mgmt.myPrint("Testing with server %s and channel %s.\n"
					 "Server is connected: %s\n"
					 "Channel is query: %s\n"
					 "Channel is joined: %s\n" % (
					 	s, c,
					 	s and s.connected,
						c and c.is_query(),
						c and c.is_channel() and c.joined))

	gui.mgmt.myPrint("cmd_connect(%s)" % (s.name,))
	# Should do nothing if we're connected
	cmdh.cmd_connect(s, c, [s.name])

	gui.mgmt.myPrint("cmd_nick(%s)" % (s.nick,))
	# Should do nothing (nick change to the same nick)
	cmdh.cmd_nick(s, c, [s.nick])

	gui.mgmt.myPrint("cmd_part(%s)" % (c.name,))
	# Part the current channel
	cmdh.cmd_part(s, c, [c.name])

	gui.mgmt.myPrint("cmd_join(%s)" % (c.name,))
	# Should join the channel
	cmdh.cmd_join(s, c, [c.name])

	gui.mgmt.myPrint("cmd_action(%s)" % (["tests","arround"],))
	# Should print "<nick> tests arround"
	cmdh.cmd_action(s, c, ["tests","arround"])

	gui.mgmt.myPrint("cmd_mode(%s)" % ([c.name, "+b", s.nick],))
	# Should ban us
	cmdh.cmd_mode(s, c, [c.name, "+b", s.nick])

	gui.mgmt.myPrint("cmd_mode(%s)" % ([c.name, "-b", s.nick],))
	# Should unban us
	cmdh.cmd_mode(s, c, [c.name, "-b", s.nick])

	gui.mgmt.myPrint("cmd_away(%s)" % ("i am away now",))
	# Should set us away with the message "i am away nao"
	cmdh.cmd_away(s, c, ["i","am","away","nao"])

	gui.mgmt.myPrint("cmd_back()")
	# Should set us back
	cmdh.cmd_back(s, c, [])

	gui.mgmt.myPrint("cmd_ctcp(%s)" % ([s.nick, "CTCPOHAI"],))
	# should send us a CTCP message with content "CTCPOHAI"
	cmdh.cmd_ctcp(s, c, [s.nick, "CTCPOHAI"])

	gui.mgmt.myPrint("cmd_names(%s)" % (c.name,))
	# should print the NAMES list of the given channel
	cmdh.cmd_names(s, c, [c.name])

	gui.mgmt.myPrint("cmd_notice(%s)" % ([s.nick, "NOTICEOHAI"],))
	# Should send us a notice with the content "NOTICEOHAI"
	cmdh.cmd_notice(s, c, [s.nick, "NOTICEOHAI"])

	gui.mgmt.myPrint("cmd_message(%s)" % ([s.nick, "MESSAGEOHAI"],))
	# Should send us a message with the content "MESSAGEOHAI"
	cmdh.cmd_message(s, c, [s.nick, "MESSAGEOHAI"])

	gui.mgmt.myPrint("cmd_list(%s)" % (c.name,))
	# Should LIST the current channel
	cmdh.cmd_list(s, c, [c.name])

	gui.mgmt.myPrint("cmd_stop_list()")
	# Should do nothing or stop the listing from above (if still running)
	cmdh.cmd_stop_list(s, c, [])

	gui.mgmt.myPrint("RAW")
	# Should send a raw PRIVMSG to ourself
	cmdh.cmd_raw(s, c, ["PRIVMSG","",":",s.nick,"","RAWOHAI"])

	gui.mgmt.myPrint("cmd_whois(%s)" % (s.nick,))
	# Trigger whois on ourself
	cmdh.cmd_whois(s, c, [s.nick])

	gui.mgmt.myPrint("cmd_clear()")
	# Should clear all outputs
	cmdh.cmd_clear(s, c, [])

	# XXX: Not tested here:
	# - quit
	# - oper
	# - kick
	# - topic
	# - nickserv



"""

Channel / Query context menu at 4. position
[ ] Sound on message receive

Configuration dialog:
- Position of the entry 				[menu_position]
- Which sound to play 					[filename]
- Use system speaker instead of a file 	[use_speaker]
- Use a command instead of a file       [beep_command]

GStreamer as playing backend.

"""

import sushi
import tekka
from tekka import com
import gtk
import subprocess

from gettext import gettext as _

DEFAULT_POSITION = 8

def tab_id_str(serverName, tabName):
	if not tabName: return serverName.lower()
	return "%s_%s" % (serverName.lower(), tabName.lower())

def tab_id(tab):
	if tab.is_server():
		return tab.name.lower()
	return "%s_%s" % (tab.server.name.lower(),tab.name.lower())


def create_menu_entry(pointedTab, echanged):
	entry = gtk.CheckMenuItem(label=_("Play sound on message"))
	entry.connect("toggled", echanged, pointedTab)
	return entry


def beep_speaker():
	""" tries several beep methods for
		different platforms and frameworks
		and returns if one method was
		successful.
	"""
	try:
		import winsound
		return winsound.Beep(30, 1)
	except:
		pass

	try:
		import MacOS
		# not in 64 bit
		return MacOS.SysBeep()
	except:
		pass

	try:
		return gtk.gdk.beep()
	except:
		pass

	try:
		tty = file("/dev/tty", "w")
		tty.write(chr(7))
		tty.close()
		return None
	except:
		pass


def beep_sound(file):
	if not file:
		return

	try:
		import winsound
		return winsound.PlaySound(
				file, winsound.SND_FILENAME|winsound.SND_ASYNC)
	except:
		pass

	# TODO: WAV playing with GStreamer


def setup_menu_wrapper(sound_plugin, entry_value_changed):
	# entry_value_changed: callback

	origin_get_menu = tekka.menus.servertree_menu.ServerTreeMenu.get_menu

	def new_get_menu(self, pointedTab):

		entry = create_menu_entry(pointedTab, entry_value_changed)
		entry.show()

		entry.set_active(eval(sound_plugin.get_config(
				"beep_%s" % (tab_id(pointedTab)), "False")))

		position = int(
				sound_plugin.get_config(
				"menu_position", default=str(DEFAULT_POSITION)))

		menu = origin_get_menu(self, pointedTab)

		menu.insert(entry, position)
		return menu

	tekka.menus.servertree_menu.ServerTreeMenu.get_menu = new_get_menu


plugin_info = (
	"Plays an optional sound on every message",
	"0.9",
	"Marian Tietz")


plugin_options = (
	("filename", _("File to play"), sushi.TYPE_STRING, ""),
	("use_speaker", _("Make a beep with the speaker"), sushi.TYPE_BOOL,
			True),
	("beep_command", _("Use a command to beep."), sushi.TYPE_STRING, ""),
	("menu_position", _("The position in the menu"), sushi.TYPE_NUMBER,
			DEFAULT_POSITION)
)


class sound(sushi.Plugin):

	def __init__(self):
		sushi.Plugin.__init__(self, "sound")

		setup_menu_wrapper(
			sound_plugin=self,
			entry_value_changed=self._entry_value_cb)

		self.connect_signal("message", self._message_cb)


	def unload(self):

		self.disconnect_signal("message", self._message_cb)


	def beep(self):

		cmd = self.get_config("beep_command")

		if type(cmd) == str and cmd != "":
			subprocess.Popen(cmd.split(" "))
			return

		v = self.get_config("use_speaker", default="true")

		if v.lower() == "true":

			beep_speaker()

		else:

			beep_sound(self.get_config("filename"))



	def _entry_value_cb(self, entry, tab):
		"""
		set config value beep_<tab identifier> to
		the value of the entry.
		True -> Beep on new messages,
		False -> Silence!
		"""

		self.set_config(
				"beep_%s" % (tab_id(tab)),
				str(entry.get_active()))


	def _message_cb(self, time, server, from_str, target, msg):

		if com.parse_from(from_str)[0] == self.get_nick(server):
			return

		if (self.get_config(
				"beep_%s" % (tab_id_str(server, target))) == "True"):

			self.beep()




# coding:utf-8
import gtk

from gettext import gettext as _

from .. import gui

class ErrorDialog(gtk.Dialog):

	def __init__(self, message):

		super(ErrorDialog,self).__init__(
			parent = gui.widgets.get_object("main_window"),
			title = _("Error occured"),
			buttons = (gtk.STOCK_CLOSE, gtk.RESPONSE_CLOSE))

		self.set_default_size(400,300)

		self.error_label = gtk.Label()
		self.error_label.set_properties(width_chars=50,
										wrap=True,
										xalign=0.0)
		self.error_label.set_markup(_(
			"<span size='larger' weight='bold'>Don't Panic!</span>\n\n"
			"An error occured – we apologize for that. "
			"Feel free to submit a bug report at "
			"<a href=\"https://bugs.launchpad.net/sushi\">"
			"https://bugs.launchpad.net/sushi</a>."))

		self.tv = gtk.TextView()
		self.tv.get_buffer().set_text(message)

		self.sw = gtk.ScrolledWindow()
		self.sw.set_properties(
			shadow_type=gtk.SHADOW_ETCHED_IN,
			hscrollbar_policy=gtk.POLICY_AUTOMATIC,
			vscrollbar_policy=gtk.POLICY_AUTOMATIC)
		self.sw.add(self.tv)

		self.vbox_inner = gtk.VBox()
		self.vbox_inner.set_property("border-width", 6)

		hbox = gtk.HBox()
		hbox.set_property("border-width", 6)
		hbox.set_spacing(12)

		hbox.pack_start(gtk.image_new_from_stock(
				gtk.STOCK_DIALOG_INFO, gtk.ICON_SIZE_DIALOG),
			expand=False)
		hbox.pack_end(self.error_label)

		align = gtk.Alignment()
		align.add(hbox)
		align.set_padding(0,6,0,0)

		self.vbox_inner.pack_start(align, expand=False)
		self.vbox_inner.pack_start(self.sw)

		self.vbox.pack_start(self.vbox_inner)

	def set_message(self, msg):
		self.tv.get_buffer().set_text(msg)





# coding: UTF-8
"""
Copyright (c) 2008 Marian Tietz
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
"""

import gtk
from ..com import sushi
from .. import gui


def setup():
	pass


def get_configurator(ctype, key, server):

	def bool_configurator(key, server):
		def apply_value(*arg):
			state = str(arg[0].get_active())
			sushi.server_set(server, "server", key, state.lower())
		return apply_value

	def text_configurator(key, server):
		def apply_value(*arg):
			s = arg[0].get_text()
			sushi.server_set(server, "server", key, s)
		return apply_value

	def file_configurator(key, server):
		def apply_value(fileChooser, *user):
			s = fileChooser.get_filename()
			sushi.server_set(server, "server", key, s)
		return apply_value

	if ctype == "bool":
		return bool_configurator(key, server)
	elif ctype == "text":
		return text_configurator(key, server)
	elif ctype == "file":
		return file_configurator(key, server)

	return None


def run(server):

	widgets =  gui.builder.load_dialog("server")

	def dialog_response_cb(dialog, response_id):
		update_commandList()
		dialog.destroy()

	def update_commandList():
		widget = widgets.get_object("commandList")
		list = [i[0].get_text() for i in widget.get_widget_matrix() if i[0].get_text()]
		sushi.server_set_list(server, "server", "commands", list)

	def update_ssl_cert_file_chooser(active):
		widgets.get_object("sslCertFileChooser").set_sensitive(active)

		if not active:
			sushi.server_set(server, "server", "ssl_db", "")

	bsignals = {
		"commandList_row_added_cb":
					lambda w,*x: update_commandList(),
		"commandList_row_removed_cb":
					lambda w,*x: update_commandList(),
		"useCustomCertificateCheckButton_toggled_cb":
					lambda w,*x: update_ssl_cert_file_chooser(w.get_active()),
		"sslCheckButton_toggled_cb":
					lambda w,*x: widgets.get_object("useCustomCertificateCheckButton").set_sensitive(w.get_active()),
	}

	# connect signals before activating anything from server config
	# so that UI elements can be toggled by reading from config.
	widgets.connect_signals(bsignals)

	# This button only exists for the 'Add server' dialog. Get rid of it.
	widgets.get_object("addButton").destroy()

	types = {"address":"text", "port":"text", "nick":"text",
		"name":"text", "nickserv":"text", "autoconnect":"bool",
				"nickserv_ghost":"bool", "ssl":"bool", "ssl_db":"file"}

	signals = {
		"addressEntry": {
			"key":"address",
			"signals":("focus-out-event", "activate")
		},
		"portEntry": {
			"key":"port",
			"signals":("focus-out-event", "activate")
		},
		"nickEntry": {
			"key":"nick",
			"signals":("focus-out-event", "activate")
		},
		"nameEntry": {
			"key":"name",
			"signals":("focus-out-event", "activate")
		},
		"nickservEntry": {
			"key":"nickserv",
			"signals": ("focus-out-event", "activate")
		},
		"autoConnectCheckButton": {
			"key":"autoconnect",
			"signals":("toggled",),
		},
		"nickservGhostCheckButton": {
			"key":"nickserv_ghost",
			"signals":("toggled",)
		},
		"sslCheckButton": {
			"key":"ssl",
			"signals":("toggled",)
		},
		"sslCertFileChooser": {
			"key":"ssl_db",
			"signals":("file-set",)
		},
	}

	for key in signals:
		c_type = types[signals[key]["key"]]
		widget = widgets.get_object(key)

		configurator = get_configurator(c_type, signals[key]["key"], server)

		for signal in signals[key]["signals"]:
			widget.connect(signal, configurator)

		value = sushi.server_get(server, "server", signals[key]["key"])

		if c_type == "text":
			widget.set_text(value)
		elif c_type == "bool":
			widget.set_active(value == "true")
		elif c_type == "file":
			widget.set_filename(value)

	if sushi.server_get(server, "server", "ssl_db") != "":
		widgets.get_object("useCustomCertificateCheckButton").set_active(True)

	# fill the command list with the existing commands
	commandList = widgets.get_object("commandList")

	for i, command in enumerate(sushi.server_get_list(server, "server", "commands")):
		commandList.get_widget_matrix()[i][0].set_text(command)
		commandList.add_row()

	dialog = widgets.get_object("server")
	dialog.connect("response", dialog_response_cb)
	dialog.show_all()







"""
Copyright (c) 2009 Marian Tietz
Copyright (c) 2006-2007  David Trowbridge

Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including
without limitation the rights to use, copy, modify, merge, publish,
distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to
the following conditions:

The above copyright notice and this permission notice shall be included
in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"""


import gtk
from math import sqrt, fabs

# actually, this should be an enum but
# there is no need for an enum, i think...
colors = (
	CONTRAST_COLOR_AQUA,
	CONTRAST_COLOR_BLACK,
	CONTRAST_COLOR_BLUE,
	CONTRAST_COLOR_BROWN,
	CONTRAST_COLOR_CYAN,
	CONTRAST_COLOR_DARK_BLUE,
	CONTRAST_COLOR_DARK_GREEN,
	CONTRAST_COLOR_DARK_GREY,
	CONTRAST_COLOR_DARK_RED,
	CONTRAST_COLOR_GREEN,
	CONTRAST_COLOR_GREY,
	CONTRAST_COLOR_LIGHT_BLUE,
	CONTRAST_COLOR_LIGHT_BROWN,
	CONTRAST_COLOR_LIGHT_GREEN,
	CONTRAST_COLOR_LIGHT_GREY,
	CONTRAST_COLOR_LIGHT_RED,
	CONTRAST_COLOR_MAGENTA,
	CONTRAST_COLOR_ORANGE,
	CONTRAST_COLOR_PURPLE,
	CONTRAST_COLOR_RED,
	CONTRAST_COLOR_VIOLET,
	CONTRAST_COLOR_WHITE,
	CONTRAST_COLOR_YELLOW,
	CONTRAST_COLOR_LAST,
) = range(24)

color_regions = [
	[40.0,  60.0, -100.0, -80.0,  -10.0,  20.0], # CONTRAST_COLOR_AQUA
	[ 0.0,  30.0,    0.0,   0.0,    0.0,   0.0], # CONTRAST_COLOR_BLACK
	[25.0,  35.0, -100.0,   0.0, -100.0, -50.0], # CONTRAST_COLOR_BLUE
	[30.0,  60.0,   30.0,  50.0,   70.0, 100.0], # CONTRAST_COLOR_BROWN
	[50.0,  65.0, -100.0, -30.0, -100.0, -50.0], # CONTRAST_COLOR_CYAN
	[ 0.0,  20.0,  -40.0,  50.0, -100.0, -60.0], # CONTRAST_COLOR_DARK_BLUE
	[20.0,  35.0, -100.0, -70.0,   60.0, 100.0], # CONTRAST_COLOR_DARK_GREEN
	[20.0,  40.0,    0.0,   0.0,    0.0,   0.0], # CONTRAST_COLOR_DARK_GREY
	[10.0,  40.0,   90.0, 100.0,   70.0, 100.0], # CONTRAST_COLOR_DARK_RED
	[15.0,  40.0, -100.0, -80.0,   80.0, 100.0], # CONTRAST_COLOR_GREEN
	[35.0,  60.0,    0.0,   0.0,    0.0,   0.0], # CONTRAST_COLOR_GREY
	[40.0,  50.0, -100.0,   0.0, -100.0, -60.0], # CONTRAST_COLOR_LIGHT_BLUE
	[60.0,  75.0,   30.0,  50.0,   80.0, 100.0], # CONTRAST_COLOR_LIGHT_BROWN
	[80.0,  90.0, -100.0, -70.0,   70.0, 100.0], # CONTRAST_COLOR_LIGHT_GREEN
	[50.0,  80.0,    0.0,   0.0,    0.0,   0.0], # CONTRAST_COLOR_LIGHT_GREY
	[55.0,  65.0,   80.0,  90.0,   75.0, 100.0], # CONTRAST_COLOR_LIGHT_RED
	[40.0,  55.0,   90.0, 100.0,  -50.0,   0.0], # CONTRAST_COLOR_MAGENTA
	[65.0,  80.0,   20.0,  65.0,   90.0, 100.0], # CONTRAST_COLOR_ORANGE
	[35.0,  45.0,   85.0, 100.0,  -90.0, -80.0], # CONTRAST_COLOR_PURPLE
	[40.0,  50.0,   80.0, 100.0,   75.0, 100.0], # CONTRAST_COLOR_RED
	[70.0,  95.0,   90.0, 100.0, -100.0,   0.0], # CONTRAST_COLOR_VIOLET
	[75.0, 100.0,    0.0,   0.0,    0.0,   0.0], # CONTRAST_COLOR_WHITE
	[90.0, 100.0,    5.0,  15.0,   92.5, 105.0], # CONTRAST_COLOR_YELLOW
]

def srgb_to_xyz_g (K):
	a = 0.055
	gamma = 2.4
	if K > 0.04045:
		return ((K+a)/(1+a))**gamma
	else:
		return K / 12.92

def xyz_to_lab_f (t):
	if (t > 0.008856):
		return t ** (1/3.0)
	else:
		return 7.787*t + 16/116.0

def rgb_to_lab (R, G, B):
	""" returns (L,a,b) """
	Xn = 0.93819
	Yn = 0.98705
	Zn = 1.07475

	gr = srgb_to_xyz_g(R / 65535.0)
	gg = srgb_to_xyz_g(G / 65535.0)
	gb = srgb_to_xyz_g(B / 65535.0)

	x = 0.412424 * gr + 0.357579 * gg + 0.180464 * gb
	y = 0.212656 * gr + 0.715158 * gg + 0.072186 * gb
	z = 0.019332 * gr + 0.119193 * gg + 0.950444 * gb

	fy = xyz_to_lab_f(y / Yn)

	L = 116 * fy - 16
	a = 500 * (xyz_to_lab_f(x / Xn) - fy)
	b = 200 * (fy - xyz_to_lab_f(z / Zn))
	return (L,a,b)

def xyz_to_srgb_C (K):
	a = 0.055
	gamma = 2.4

	if (K > 0.00304):
		return (1 + a) * (K ** (1.0 / gamma)) - a
	else:
		return K * 12.92

def CLAMP(x, low, high):
	if x > high:
		return high
	elif x < low:
		return low
	else:
		return x

def lab_to_rgb (L, a, b):
	""" returns (R, G, B) """
	Xn = 0.93819
	Yn = 0.98705
	Zn = 1.07475

	fy = (L + 16) / 116
	fx = fy + a / 500
	fz = fy - b / 200
	delta = 6.0 / 29
	delta2 = delta ** 2

	if (fx > delta): x = Xn * fx ** 3
	else:            x = (fx - 16.0/116) * 3 * delta2 * Xn

	if (fy > delta): y = Yn * fy ** 3
	else:            y = (fy - 16.0/116) * 3 * delta2 * Yn

	if (fz > delta): z = Zn * fz ** 3
	else:            z = (fz - 16.0/116) * 3 * delta2 * Zn

	rs =  3.2410 * x - 1.5374 * y - 0.4986 * z
	gs = -0.9692 * x + 1.8760 * y + 0.0416 * z
	bs =  0.0556 * x - 0.2040 * y + 1.0570 * z

	R = CLAMP(int(round(xyz_to_srgb_C(rs) * 65535)), 0, 65535)
	G = CLAMP(int(round(xyz_to_srgb_C(gs) * 65535)), 0, 65535)
	B = CLAMP(int(round(xyz_to_srgb_C(bs) * 65535)), 0, 65535)

	return (R, G, B)

def lab_distance (La, aa, ba, Lb, ab, bb):
	dL = fabs(Lb - La)
	da = fabs(ab - aa)
	db = fabs(bb - ba)
	return sqrt(dL*dL + da*da + db*db)

def contrast_render_foreground_color (background, color):

	"""
	* contrast_render_foreground_color
	* @background: the background color (GdkColor)
	* @color: the desired foreground color (one of those constants on the top)
	*
	* Creates a specific color value for a foreground color, optimizing for
	* maximum readability against the background.
   	"""

	points = [[0 for n in range(3)] for n in range(8)]

	(L, a, b) = rgb_to_lab(background.red, background.green, background.blue)

	points[0][0] = color_regions[color][0]; points[0][1] = color_regions[color][2]; points[0][2] = color_regions[color][4];
	points[1][0] = color_regions[color][0]; points[1][1] = color_regions[color][2]; points[1][2] = color_regions[color][5];
	points[2][0] = color_regions[color][0]; points[2][1] = color_regions[color][3]; points[2][2] = color_regions[color][4];
	points[3][0] = color_regions[color][0]; points[3][1] = color_regions[color][3]; points[3][2] = color_regions[color][5];
	points[4][0] = color_regions[color][1]; points[4][1] = color_regions[color][2]; points[4][2] = color_regions[color][4];
	points[5][0] = color_regions[color][1]; points[5][1] = color_regions[color][2]; points[5][2] = color_regions[color][5];
	points[6][0] = color_regions[color][1]; points[6][1] = color_regions[color][3]; points[6][2] = color_regions[color][4];
	points[7][0] = color_regions[color][1]; points[7][1] = color_regions[color][3]; points[7][2] = color_regions[color][5];

	max_dist = 0
	max_color = 0

	for i in range(8):
		dist = lab_distance(L, a, b, points[i][0], points[i][1], points[i][2])

		if dist > max_dist:
			max_dist = dist
			max_color = i

	"""
	If the luminosity distance is really short, extend the vector further
	out.  This may push it outside the bounds of the region that a color
	is specified in, but it keeps things readable when the background and
	foreground are really close.
	"""
	ld = fabs( L - points[max_color][0] )
	cd = sqrt( fabs(a - points[max_color][1]) ** 2 + fabs(b - points[max_color][2]) ** 2 )

	if (ld < 10.0) and (cd < 60.0):
		dL = points[max_color][0] - L
		da = points[max_color][1] - a
		db = points[max_color][2] - b
		points[max_color][0] = L + (dL * 4.0)
		points[max_color][1] = a + (da * 1.5)
		points[max_color][2] = b + (db * 1.5)

	red, green, blue = lab_to_rgb(
				points[max_color][0],
				points[max_color][1],
				points[max_color][2])

	return gtk.gdk.Color (red, green, blue)

if __name__ == "__main__":
	import sys

	if len(sys.argv) < 2:
		sys.exit(1)

	cnames = [
		"aqua","black","blue","brown","cyan","darkblue",
		"darkgreen","darkgray","darkred","green","gray",
		"lightblue","lightbrown","lightgreen","lightgrey",
		"lightred","magenta","orange","purple","red",
		"violet","white","yellow","last"]

	c = gtk.gdk.Color(sys.argv[1])
	for i in range(23):
		r = contrast_render_foreground_color(c, i)
		print "output(%d:%s): \t%s" % (i, cnames[i], r)



#!/usr/bin/env python
# coding: UTF-8
"""
Copyright (c) 2009 Marian Tietz
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
"""

import sys
import re

"""
irc://<server>[:<port>]/[<((#|&|+)channel>|nick)>[,(isnick|needkey)]
"""

pattern = re.compile(r"irc://([\w.:]*)/(.*)")

def prompt_for_key():
	""" prompt the user for the needed channel key """
	print "UH CHANNEL KEY PL=X: "

def nick_connect(server, port, nickinfo):
	""" nickinfo can be the nick name or the usermask """
	print "Connect to server %s:%s and talk to %s" % (server, port, nickinfo)
	pass

def server_connect(server, port):
	""" server == "default" means default server,
		port == "default" means default port
	"""
	print "Connect to server %s:%s" % (server, port)
	pass

def channel_connect(server, port, channel, key):
	""" key can be "" """
	print "Connect to server %s:%s and join %s (key='%s')" % (
		server, port, channel, key)
	pass

if __name__ == "__main__":

	if len(sys.argv) != 2:
		sys.exit(1)

	input = sys.argv[1]
	match = pattern.match(input)

	if not match:
		print "No valid IRC URL."
		sys.exit(1)

	address, target = match.groups()

	address_split = address.split(":")
	if len(address_split) == 2:
		server, port = address_split

	elif len(address_split) == 1 and address_split[0]:
		server = address_split[0]
		port = "6667"

	else:
		server = "default"
		port = "default"

	target_split = target.split(",")
	if len(target_split) == 2:
		target_name, appendix = target_split
	elif len(target_split) == 1:
		target_name = target_split[0]
		appendix = ""
	else:
		print "Unknown target format."
		sys.exit(1)

	if target_name and target_name[0] in ("#","&","+"):
		# target is a channel
		key = ""
		if appendix == "isnick":
			print "Channel prefix and nick identifier. Abort."
			sys.exit(1)

		elif appendix == "needkey":
			prompt_for_key()

		else:
			if appendix:
				key = appendix

		channel_connect(server, port, target_name, key)

	elif target_name:
		if appendix != "isnick":
			print "No isnick flag set. Invalid syntax."
			sys.exit(1)

		nick_connect(server, port, target_name)

	else:
		server_connect(server, port)




import os
from tekka.helper import history

def start_test(gui):
	def dprint(*x,**y):
		gui.mgmt.myPrint(*x,**y)

	dprint("get_log_dir()")
	dprint("%s" % history.get_log_dir())
	dprint("get_available_servers()")
	dprint("%s" % history.get_available_servers())
	dprint("get_available_conversations('euIRC')")
	dprint("%s" % history.get_available_conversations('euIRC'))
	dprint("get_available_logs('euIRC','#bsdunix')")
	dprint("%s" % history.get_available_logs('euIRC', '#bsdunix'))
	dprint("get_log_data('2009-13.txt')")
	dprint("%s" % (history.get_log_date('2009-13.txt'),))

	fpath = os.path.join(history.get_log_dir(), 'euIRC', '#bsdunix',
						'2009-03.txt')
	dprint("parse_day_offsets(%s)" % (fpath))
	dprint("%s" % (history.parse_day_offsets(file(fpath,"r")),))




# coding: UTF-8
"""
Copyright (c) 2008 Marian Tietz
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
"""

import gtk
from gettext import gettext as _

from .. import config
from .. import gui
from ..com import sushi
from ..helper.singleton import SingletonMeta

from ..lib import key_dialog
from ..lib import topic_dialog


class ServerTreeMenu(object):

	__metaclass__ = SingletonMeta

	def __init__(self):
		self.menu = None
		self.widgets = gui.builder.load_menu("serverTreeMenu")

		if not self.widgets:
			gui.mgmt.show_inline_message(
				_("Widget creation failed."),
				_("tekka failed to create the nicklist menu.\n"
				  "It's possible that there are files missing. "
				  "Check if you have appropriate permissions to "
				  "access all files needed by tekka and restart tekka."),
				dtype="error")
			return

		sigdic = {
			"serverTreeMenu_deactivate_cb" : self.deactivate_cb,

			"connectItem_activate_cb" : self.connectItem_activate_cb,
			"disconnectItem_activate_cb" : self.disconnectItem_activate_cb,
			"joinChannelItem_activate_cb" : self.joinChannelItem_activate_cb,
			"joinItem_activate_cb" : self.joinItem_activate_cb,
			"partItem_activate_cb" : self.partItem_activate_cb,
			"closeItem_activate_cb" : self.closeItem_activate_cb,
			"autoJoinItem_toggled_cb" : self.autoJoinItem_toggled_cb,
			"autoConnectItem_toggled_cb" : self.autoConnectItem_toggled_cb,
			"hideItem_activate_cb": self.hideItem_activate_cb,
			"historyItem_activate_cb" : self.historyItem_activate_cb,
			"setTopicItem_activate_cb": self.setTopicItem_activate_cb,
			"setKeyItem_activate_cb" : self.setKeyItem_activate_cb
		}

		self.widgets.connect_signals(sigdic)

		self.menu = self.widgets.get_object("serverTreeMenu")

	def get_menu(self, pointedTab):
		""" return the menu customized menu, fit to the needs of pointedTab """
		if not self.menu:
			return None

		self.current_tab = pointedTab
		self.headline = gtk.MenuItem(pointedTab.name)

		self.menu.insert(self.headline,0)
		self.menu.show_all()

		connectItem = self.widgets.get_object("connectItem")
		disconnectItem = self.widgets.get_object("disconnectItem")
		joinChannelItem = self.widgets.get_object("joinChannelItem")
		joinItem = self.widgets.get_object("joinItem")
		partItem = self.widgets.get_object("partItem")
		autoConnectItem = self.widgets.get_object("autoConnectItem")
		autoJoinItem = self.widgets.get_object("autoJoinItem")
		hideItem = self.widgets.get_object("hideItem")
		historyItem = self.widgets.get_object("historyItem")
		closeItem = self.widgets.get_object("closeItem")
		setTopicItem = self.widgets.get_object("setTopicItem")
		setKeyItem = self.widgets.get_object("setKeyItem")

		# set up visibilty of menu items for each case
		if pointedTab.is_server():
			joinItem.hide()
			partItem.hide()
			setTopicItem.hide()
			setKeyItem.hide()
			autoJoinItem.hide()
			hideItem.hide()
			historyItem.hide()

			if sushi.server_get(
			pointedTab.name, "server", "autoconnect") == "true":
				autoConnectItem.set_active(True)
			else:
				autoConnectItem.set_active(False)

			if pointedTab.connected:
				connectItem.hide()
			else:
				disconnectItem.hide()

		elif pointedTab.is_channel():
			connectItem.hide()
			disconnectItem.hide()
			autoConnectItem.hide()
			joinChannelItem.hide()

			if sushi.server_get(
			pointedTab.server.name, pointedTab.name, "autojoin") == "true":
				autoJoinItem.set_active(True)
			else:
				autoJoinItem.set_active(False)

			if pointedTab.joined:
				joinItem.hide()
			else:
				partItem.hide()

		elif pointedTab.is_query():
			autoConnectItem.hide()
			autoJoinItem.hide()
			setTopicItem.hide()
			setKeyItem.hide()
			connectItem.hide()
			disconnectItem.hide()
			joinItem.hide()
			partItem.hide()
			joinChannelItem.hide()

		return self.menu

	def deactivate_cb(self, menu):
		menu.remove(self.headline)

	def connectItem_activate_cb(self, item):
		""" Connect to the server. """
		if self.current_tab and self.current_tab.is_server():
			sushi.connect(self.current_tab.name)

	def disconnectItem_activate_cb(self, item):
		""" quit server with default quit message. """
		if self.current_tab and self.current_tab.is_server():
			sushi.quit(
				self.current_tab.name,
				config.get("chatting", "quit_message", ""))

	def joinChannelItem_activate_cb(self, item):
		""" pop up a dialog to ask which channel should be joined """
		if self.current_tab and self.current_tab.is_server():
			gui.dialogs.show_dialog("join", self.current_tab.name)

	def joinItem_activate_cb(self, item):
		""" join channel without key """
		if self.current_tab and self.current_tab.is_channel():
			sushi.join(self.current_tab.server.name,
				self.current_tab.name, "")

	def partItem_activate_cb(self, item):
		""" part channel with default part message """
		if self.current_tab and self.current_tab.is_channel():
			sushi.part(
				self.current_tab.server.name,
				self.current_tab.name,
				config.get("chatting", "part_message", ""))

	def closeItem_activate_cb(self, item):
		""" close tab. If the tab is a server emit a quit.
			If the tab is a channel, part the channel before.
		"""
		if not self.current_tab:
			return

		if self.current_tab.is_channel() and self.current_tab.joined:
			sushi.part(
				self.current_tab.server.name,
				self.current_tab.name,
				config.get("chatting", "part_message", ""))

		elif self.current_tab.is_server() and self.current_tab.connected:
			sushi.quit(
				self.current_tab.name,
				config.get("chatting", "quit_message", ""))

		gui.tabs.remove_tab(self.current_tab)
		gui.shortcuts.assign_numeric_tab_shortcuts(gui.tabs.get_all_tabs())

	def autoJoinItem_toggled_cb(self, item):
		""" set the auto join state of the tab to the state
			of the menu item. (for channels)
		"""
		if not self.current_tab or not self.current_tab.is_channel():
			return

		sushi.server_set(
			self.current_tab.server.name,
			self.current_tab.name,
			"autojoin", str(item.get_active()).lower())

	def autoConnectItem_toggled_cb(self, item):
		""" set the auto connect state of the tab to the state
			of the menu item. (for servers)
		"""
		if not self.current_tab or not self.current_tab.is_server():
			return

		sushi.server_set(self.current_tab.name,
			"server", "autoconnect", str(item.get_active()).lower())

	def hideItem_activate_cb(self, item):
		""" show up hide messages dialog """
		if not self.current_tab or self.current_tab.is_server():
			return

		gui.dialogs.show_dialog("hide", self.current_tab)

	def historyItem_activate_cb(self, item):
		""" show up history dialog for current tab. """
		if not self.current_tab or self.current_tab.is_server():
			return

		gui.dialogs.show_dialog("history", self.current_tab)

	def setTopicItem_activate_cb(self, item):
		""" show up inline dialog to change topic """
		def dialog_response_cb(dialog, id):
			if id != gtk.RESPONSE_OK:
				dialog.destroy()

		if not self.current_tab or not self.current_tab.is_channel():
			return

		d = topic_dialog.TopicDialog(
			self.current_tab.server.name,
			self.current_tab.name)
		d.connect("response", dialog_response_cb)
		gui.mgmt.show_inline_dialog(d)

	def setKeyItem_activate_cb(self, item):
		""" show up dialog for key setting """
		if not self.current_tab or not self.current_tab.is_channel():
			return

		server = self.current_tab.server.name
		channel = self.current_tab.name

		d = key_dialog.KeyDialog(server, channel)

		d.checkButton.set_property("visible", False)
		d.checkButton.set_active(True)
		d.connect("response", lambda d,i: d.destroy())

		gui.mgmt.show_inline_dialog(d)




# coding: UTF-8
"""
Copyright (c) 2009 Marian Tietz
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
"""

import urwid
import logging

import connection
import commands
import signals
import config
import tabs
import messages
import __main__

# TODO: update plugin API to current version

# XXX:  maybe better proxy both, commands and signals,
# XXX:: to catch errors in the error code.

"""
TYPE_STRING: Takes one argument (default string)
TYPE_PASSWORD: Hidden string. Takes one argument (default string)
TYPE_NUMBER: Takes one argument (default number)
TYPE_BOOL: Takes one argument (default bool value)
TYPE_CHOICE: Takes n key/value tuple and a default index.
"""
(TYPE_STRING,
 TYPE_PASSWORD,
 TYPE_NUMBER,
 TYPE_BOOL,
 TYPE_CHOICE
) = range(5)

class Plugin (object):

	def __init__(self, plugin_name):
		self.__registered_signals = {}

		self._plugin_name = plugin_name

		urwid.connect_signal(connection.sushi, "connected", self.maki_connected)
		urwid.connect_signal(connection.sushi, "disconnected", self.maki_disconnected)

	def __register_signal(self, signal, handler):
		if self.__registered_signals.has_key(signal):
			self.__registered_signals[signal].append(handler)
		else:
			self.__registered_signals[signal] = [handler]

	def __unregister_signal(self, signal, handler):
		if self.__registered_signals[signal].has_key(signal):
			try:
				i = self.__registered_signals[signal].index(handler)
			except ValueError:
				return
			del self.__registered_signals[signal][i]

	def unload(self):
		for (signal, handlers) in self.__registered_signals.items():
			for handler in handlers:
				signals.disconnect_signal(signal, handler)

	def maki_connected(self, sushi):
		pass

	def maki_disconnected(self, sushi):
		pass

	def get_bus(self):
		return connection.sushi

	def get_nick(self, server):
		server_tab = __main__.main_window.find_server(server)
		if not server_tab:
			return None

		nick = server_tab.get_nick()

		if not nick:
			return None

		return nick

	def add_command(self, command, func):
		def func_proxy (main_window, argv):
			ct = main_window.current_tab

			if not ct:
				server_name = ""
				target_name = ""
			elif type(ct) == tabs.Server:
				server_name = ct.name
				target_name = ""
			else:
				server_name = tabs.get_server(ct).name
				target_name = ct.name

			return func(server_name, target_name, argv[1:])

#		self.emit("command_add", command, func)
		return commands.add_command(command, func_proxy)

	def remove_command(self, command):
#		self.emit("command_remove", command, func)
		return commands.remove_command(command)

	def connect_signal(self, signal, func):
#		self.emit("signal_connect", signal, func)
		self.__register_signal(signal,func)
		return signals.connect_signal(signal, func)

	def disconnect_signal(self, signal, func):
#		self.emit("signal_disconnect", signal, func)
		self.__unregister_signal(signal, func)
		return signals.disconnect_signal(signal, func)

	def set_config(self, name, value):
		section = "plugin_%s" % (self._plugin_name)

		config.create_section(section)

		return config.set(section, name, value)

	def get_config(self, name, default = None):
		section = "plugin_%s" % (self._plugin_name)

		return config.get(section, name, default)

	def display_error(self, error):
		logging.error("%s: %s" % (self._plugin_name, error))
		messages.print_error("%s: %s" % (self._plugin_name,error))

	def parse_from(self, host):
		return connection.parse_from(host)






class TekkaError(object):

	"""
	Generic Error object to return in functions as last parameter
	to work around exceptions.

	Example:
	calcStuff(a,b): # returns value,err
		if a != 2:
			return None,TekkaError("a must not be 2.")
		if b < 0:
			return None,TekkaError("b must not be lower than 0")
		return a*b,None
	"""

	def __init__(self, msg):
		self.msg = msg

	def __str__(self):
		return self.msg





from .._builder import widgets

"""
Caching of the current_tab and convenience methods around it.
"""

current_path = ()

def _set_current_path(path):
	" called by the parent module on a tab switch "
	global current_path
	current_path = path


def get_current_path():
	return current_path


def get_current_tab():
	""" Returns the current tab """
	global current_path

	store = widgets.get_object("tab_store")

	try:
		return store[current_path][0]
	except (IndexError,TypeError):
		return None


def get_current_tabs():
	"""
		Returns a tuple with the server
		as parent tab and the active channel tab.

		If only a server is active, the
		second field of the tuple is None.

		Possible return values:
		(<serverTab>,<channelTab>)
		(<serverTab>,None)
		(None,None)
	"""
	global current_path

	store = widgets.get_object("tab_store")

	if not current_path:
		return None,None

	# iter could be server or channel
	try:
		iter = store.get_iter(current_path)
	except ValueError:
		# tab is already closed
		return None, None

	if not iter:
		return None, None

	pIter = store.iter_parent(iter)
	if not pIter:
		# no parent, iter is a server
		return store.get_value(iter, 0), None
	else:
		return store.get_value(pIter, 0), store.get_value(iter, 0)

	return None, None




from ..builder import widgets, build_status_icon
from ... import config
from ...lib.welcome_window import WelcomeWindow


def _show_hide(widget, show, all=False):
	if show:
		if all: widget.show_all()
		else: widget.show()
	else:
		if all: widget.hide_all()
		else: widget.hide()

def show_topic_bar(show):
	_show_hide(widgets.get_object("topic_alignment"), show, all=True)

def show_status_icon(show):
	icon = widgets.get_object("status_icon")

	if show:
		if not icon:
			build_status_icon()
			return
	else:
		if not icon:
			return
	icon.set_visible(show)

def show_status_bar(show):
	_show_hide(widgets.get_object("statusbar"), show)

def show_side_pane(show):
	_show_hide(widgets.get_object("list_vpaned"), show)

def show_general_output(show):
	_show_hide(widgets.get_object("general_output_alignment"), show,
		all=True)

def show_nicks(show):
	_show_hide(widgets.get_object("nicks_vbox"), show)

def show_welcome_screen(show):
	""" show a welcome screen while hiding general output,
		topic bar and side pane.
	"""

	# widgets to hide on welcome
	hides = (
		"show_side_pane",
		"show_topic_bar",
		"show_general_output"
	)

	for to_hide in hides:
		if config.get_bool("tekka",to_hide):
			# it should be showed, hide it if welcome
			# screen should be showed, otherwise show it
			eval(to_hide)(not show)

	if show:
		s = widgets.get_object("output_shell")

		w = WelcomeWindow()

		s.set(w)
		s.show_all()



def apply_visibility_from_config():
	""" read the config values of the widgets which can be
		hidden and apply the configured value to the widgets
	"""

	c_w = {
		"show_general_output": show_general_output,
		"show_topic_bar": show_topic_bar,
		"show_side_pane": show_side_pane,
		"show_status_bar": show_status_bar,
		"show_status_icon": show_status_icon,
		}

	for option in c_w:
		c_w[option](config.get_bool("tekka", option))






import gtk

"""

Holds the TekkaBuilder class object, initially needed for
the builder module.

Also provides a static reference to the initial instance of
the TekkaBuilder class (widgets) which is used all over
the application to access widgets.

"""

class TekkaBuilder(gtk.Builder):
	""" extended version of the gtk.Builder which supports
		adding GObject widgets to the list of storable
		objects in the builder.
	"""

	def __init__(self):
		super(TekkaBuilder,self).__init__()
		self._widgets = {}

	def get_object(self, name):
		obj = super(TekkaBuilder,self).get_object(name)
		if obj:
			return obj
		if self._widgets.has_key(name):
			return self._widgets[name]
		return None

	def add_object(self, object, name):
		if (super(TekkaBuilder,self).get_object(name)
		or self._widgets.has_key(name)):
			raise ValueError, "Object '%s' already in Builder." % (name)
		self._widgets[name] = object
		return True

	def remove_object(self, name):
		if self._widgets.has_key(name):
			del self._widgets[name]
			return True
		return False

widgets = TekkaBuilder()



import gobject
import time

from ... import config

from ...helper import color
from ...typecheck import types

from . import tab
from . import util

from .messages import *


class TekkaChannel(tab.TekkaTab):

	@types(switch=bool)
	def _set_joined(self, switch):
		self._joined = switch
		self.emit("joined", switch)
		self.emit("new_markup")


	@types(topic=basestring)
	def _set_topic(self, topic):
		self._topic = topic
		self.emit("topic", topic)


	joined = property(lambda x: x._joined, _set_joined)
	topic = property(lambda x: x._topic, _set_topic)


	def __init__(self, name, server, textview=None,
		nicklist=None, topic="", topicsetter=""):

		super(TekkaChannel,self).__init__(name, textview)

		self.nickList = nicklist        # nick list object
		self.topic = topic              # topic string
		self.topicSetter = topicsetter  # the nick of the topic setter
		self.joined = False             # status flag

		self.server = server            # the server name string


	def is_channel(self):
		return True


	def markup(self):
		italic = False
		bold = False
		foreground = None

		base = self.name

		if not self.joined:
			base = "<span strikethrough='true'>"+base+"</span>"

		if ACTION in self.newMessage:
			italic = True

		if MESSAGE in self.newMessage:
			if config.get_bool("colors","color_new_message"):
				foreground = util._markup_color("new_message")
			bold = True

		if (HIGHMESSAGE in self.newMessage
		and HIGHACTION in self.newMessage):
			foreground = util._markup_color("new_highlightmessage")
			bold = True
		elif HIGHMESSAGE in self.newMessage:
			foreground = util._markup_color("new_highlightmessage")
			bold = True
		elif HIGHACTION in self.newMessage:
			foreground = util._markup_color("new_highlightaction")
			bold = True

		markup = "<span "
		if italic:
			markup += "style='italic' "

		if bold:
			markup += "weight='bold' "

		if foreground:
			markup += "foreground='%s'" % foreground

		markup += ">%s</span>" % base

		return markup


	def write(self, timestamp, message, msgtype=MESSAGE,
	no_general_output=False, **kwargs):
		""" write [<timestamp>] <message> to this tab's buffer """

		timestring = time.strftime(
					config.get("chatting", "time_format", "%H:%M"),
					time.localtime(timestamp))

		cString = color.colorize_message(msgtype, message)

		outputString = "[%s] %s" % (timestring, cString)

		buffer = self.window.textview.get_buffer()
		buffer.insert_html(buffer.get_end_iter(), outputString, **kwargs)

		if not self.is_active():

			if (config.get_bool("tekka", "show_general_output")
			and not no_general_output):

				# write it to the general output, also
				util._write_to_general_output(
					msgtype,
					timestring,
					self,
					message)

		def notify():
			self.set_new_message(msgtype)
			return False

		gobject.idle_add(notify)

""" Joined status changed. status as parameter """
gobject.signal_new(
	"joined",
	TekkaChannel,
	gobject.SIGNAL_ACTION,
	gobject.TYPE_NONE,
	(gobject.TYPE_BOOLEAN,))

""" Topic changed. topic as parameter """
gobject.signal_new(
	"topic",
	TekkaChannel,
	gobject.SIGNAL_ACTION,
	gobject.TYPE_NONE,
	(gobject.TYPE_STRING,))




# coding: UTF-8
"""
Copyright (c) 2009 Marian Tietz
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
"""

"""
plugin interface

load(filename)
-> imp(filename)
-> plugins[filename] = <filename>.filename(pluginInterface)

unload(filename)
-> plugins[filename].unload()
-> del plugins[filename]
"""

import gobject
import os
import imp
import sys
import logging
import traceback

import config
from messages import print_error

_module_prefix = "nigiriplugin_"
_plugins = {}

(
	PLUGIN_MODNAME,
	PLUGIN_MODULE,
	PLUGIN_INSTANCE,
	PLUGIN_INFO_LENGTH
) = range(4)

def strip_suffix(filename):
	""" foo.py -> foo """
	return os.path.split(filename)[-1].split(".")[0]

def get_plugin_config_section(filename):
	return "plugin_"+strip_suffix(filename)

def is_loaded(filename):
	""" returns True if the plugin is loaded, otherwise False """
	return _plugins.has_key(filename)

def _register_plugin(filename, modname, module, instance):
	""" add plugin to local plugin dict, key is filename """
	global _plugins
	if _plugins.has_key(filename):
		return False
	_plugins[filename] = (modname, module, instance)
	return True

def _unregister_plugin(filename):
	""" remove plugin from local plugin dict """
	global _plugins
	if not _plugins.has_key(filename):
		return False
	del _plugins[filename]
	return True

def _find_module(filename):
	""" wrapper for imp.find_module.
		Searches for the module named after
		filename in the configured search
		path (tekka, plugin_dirs).
	"""
	# get the name of the module to search for
	name = strip_suffix(filename)

	mod_info = None
	try:
		mod_info = imp.find_module(
			name, config.get_list("nigiri","plugin_dirs",[]))

	except ImportError, e:
		print_error("Error while finding module for '%s'" % (filename))
		return None

	if not mod_info:
		print_error("No plugin found for filename '%s'" % (filename))
		return None

	return mod_info

def _load_module(filename, mod_info):
	""" wrapper for imp.load_module.
		Returns a tuple with the identifying
		name of the loaded module and the
		module itself if loading was successful.
		Otherwise the function returns (None,None).
	"""
	name = strip_suffix(filename)
	plugin = None
	modname = _module_prefix + name

	try:
		plugin = imp.load_module(modname, *mod_info)

	except ImportError,e:
		print_error("Failure while loading plugin '%s': %s" % (name, e))

	try:
		mod_info[0].close()
	except (IndexError,AttributeError):
		pass

	return modname, plugin

def _unload_module(name):
	""" unload the module loaded by imp.load_module
		by deleting it's reference from sys.modules
	"""
	del sys.modules[name]


def load(filename):
	""" load a module named after filename in the
		configured search path (tekka, plugin_dirs),
		instance the class named after the module
		(np for np.py) and register the bunch.

		On error this function calls print_error()
		with an specific error message and returns
		False. Otherwise this function returns True.
	"""
	global _plugins

	if _plugins.has_key(filename):
		print_error("A plugin with that name is already loaded.")
		return False

	mod_info = _find_module(filename)

	if not mod_info:
		return False

	modname, plugin = _load_module(filename, mod_info)
	classname = strip_suffix(filename)

	if not plugin:
		_unload_module(modname)
		return False

	try:
		instance = eval ("plugin.%s()" % (classname))

	except BaseException,e:
		logging.debug("plugin %s: %s" % (filename, traceback.format_exc(e)))
		print_error("Error while instancing plugin: %s" % (e))
		_unload_module(modname)
		return False

	if not _register_plugin(filename, modname, plugin, instance):
		_unload_module(modname)
		return False
	return True

def unload(filename):
	""" Call unload() in the plugin instance,
		unload the module and unregister it.
		On success this function returns True,
		otherwise it returns False.
	"""
	global _plugins

	try:
		entry = _plugins[filename]
	except KeyError:
		print_error("Failed to unload plugin '%s', does not exist." % (filename))
		return False

	# tell the instance, it's time to go
	try:
		entry[PLUGIN_INSTANCE].unload()
	except:
		pass

	_unload_module(entry[PLUGIN_MODNAME])

	# unregister locally
	return _unregister_plugin(filename)

def _check_info_tuple(info):
	if (type(info) != tuple
		or len(info) != PLUGIN_INFO_LENGTH
		or len([n for n in info if type(n) != str])):
		return False
	return True

def get_info(filename):
	""" return the plugin info from the plugin.
		If any error occurs, this function
		returns None, else it returns a tuple.

		tuple returned:
		(<description>, <version>, <author>)
	"""
	global _plugins

	if _plugins.has_key(filename):
		try:
			info = _plugins[filename][PLUGIN_MODULE].plugin_info
		except:
			return None

		if not _check_info_tuple(info):
			return None
		return info
	else:
		name = strip_suffix(filename)
		mod_info = _find_module(filename)

		if not mod_info:
			return None

		modname, plugin = _load_module(filename, mod_info)

		if not plugin:
			return None

		try:
			info = plugin.plugin_info
		except AttributeError:
			_unload_module(modname)
			return None

		# return None if the attribute is not a tuple
		# or any item in the tuple is not a string
		if not _check_info_tuple(info):
			_unload_module(modname)
			return None

		_unload_module(modname)

		return info

	return None

def _check_options_tuple(options):
	if type(options) != tuple:
		return False

	for x in options:
		# name, label and type are required
		if type(x) != tuple or len(x) < 3:
			return False

	return True

def get_options(filename):
	""" Return the configurable options and the input
		types associated with every option.
		If no options are found, None is returned.

		e.g. ( 	("pass", "some label", sushi.TYPE_PASSWORD, ""),
				("name", "another label", sushi.TYPE_ENTRY, "default"))
	"""
	global _plugins

	def retrieve_options(plugin):
		try:
			options = plugin.plugin_options
		except AttributeError:
			return None

		if not _check_options_tuple(options):
			return None

		return options

	if _plugins.has_key(filename):
		return retrieve_options(_plugins[filename][PLUGIN_MODULE])

	else:
		mod_info = _find_module(filename)

		if not mod_info:
			return None

		modname, plugin = _load_module(filename, mod_info)

		if not plugin:
			return None

		options = retrieve_options(plugin)

		_unload_module(modname)

		return options

	return None

def load_autoloads():
	autoloads = config.get("autoload_plugins").items()
	for opt,filename in autoloads:
		print "autoloading '%s'" % (filename)
		if not load(filename):
			print_error("Failed to load plugin '%s' automatically, "\
				"removing from list." % (filename))
			config.unset("autoload_plugins",opt)




import gtk

class CustomColorButton(gtk.ColorButton):

	""" A ColorButton which leaves it to the developer what
		to open after a click.
	"""

	__gtype_name__ = "CustomColorButton"

	def do_clicked(self):
		pass

	def do_color_set(self):
		pass




# coding: UTF-8
"""
Copyright (c) 2009 Marian Tietz
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
"""

import gtk
import gobject
from gobject import TYPE_STRING
from gettext import gettext as _

from .. import signals
from .. import com

class WhoisDialog(gtk.Dialog):

	def __init__(self, server, nick):
		gtk.Dialog.__init__(self,
			flags=gtk.DIALOG_DESTROY_WITH_PARENT,
			buttons=(gtk.STOCK_CLOSE, gtk.RESPONSE_CLOSE))

		self.set_default_size(350, 200)

		self.end = False

		self.treeview = self._setup_treeview()

		self.scrolled_window = gtk.ScrolledWindow()
		self.scrolled_window.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
		self.scrolled_window.add(self.treeview)

		self.get_content_area().add(self.scrolled_window)

		self.set_data(server, nick)


	def _setup_treeview(self):
		treeview = gtk.TreeView()
		treeview.set_model(gtk.ListStore(TYPE_STRING))

		renderer = gtk.CellRendererText()
		column = gtk.TreeViewColumn(
			"Data", renderer, text=0)

		treeview.append_column(column)

		return treeview

	def set_data(self, server, nick):
		self.nick = nick
		self.server = server
		self.set_title(_("Whois on %(server)s" % {
			"server":server}))

		label = gtk.Label()
		label.set_use_underline(False)
		label.set_text(_("Whois data of %(nick)s" % {
				"nick":nick}))
		label.show()

		self.treeview.get_column(0).set_widget(label)

	def whois_input(self, time, server, nick, message):
		# message == "" -> EOL
		if self.end:
			self.treeview.get_model().clear()
			self.end = False

		if message:
			self.treeview.get_model().append(row=(message,))

		else:
			self.end = True

diag = None

def dialog_response_cb(dialog, id):
	if id in (gtk.RESPONSE_NONE, gtk.RESPONSE_CLOSE):
		global diag
		signals.disconnect_signal("whois", dialog.whois_input)

		diag = None
		dialog.destroy()

def loading_timeout_cb(dialog):
	if (dialog.treeview.get_model()
	and len(dialog.treeview.get_model()) <= 1):
		dialog.end = True
		dialog.whois_input(0, "", "", _("No data received so far. Are you still connected?"))
	return False

def run(server, nick):
	global diag

	if not diag:
		diag = WhoisDialog(server, nick)
		diag.connect("response", dialog_response_cb)

		signals.connect_signal("whois", diag.whois_input)

	else:
		diag.set_data(server, nick)

	com.sushi.whois(server, nick)
	diag.whois_input(0, "", "",_("Loading..."))
	diag.end = True

	gobject.timeout_add(20000, loading_timeout_cb, diag)

	diag.show_all()

def setup():
	pass



# coding: UTF-8
"""
Copyright (c) 2009 Marian Tietz
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
"""

import gtk
import gobject
import pango

from .. import gui
from .. import config

widgets = None

def addCategory(store, category):
	""" add parent item to store and return the treeiter """
	iter = store.append(None, (category,"",""))

	return iter

def fillConfigView():
	"""
	fill config treeview with categories,
	options/values and defaults
	"""
	configView = widgets.get_object("configView")
	configStore = widgets.get_object("pref_store")

	for category in ("tekka","colors","chatting","dcc","colors"):
		cDict = config.get(category, default=None)

		if not cDict:
			continue

		iter = addCategory(configStore, category)

		if not iter:
			continue

		for (key, value) in cDict.items():
			default = str(config.get_default(category, key))
			configStore.append(iter, row=(key, value, default))

def renderOption(column, renderer, model, iter):
	if not model.iter_parent(iter):
		# category row, markup bold
		renderer.set_property("weight", pango.WEIGHT_BOLD)
	else:
		renderer.set_property("weight", pango.WEIGHT_NORMAL)

def renderValue(column, renderer, model, iter):
	if not model.iter_parent(iter):
		renderer.set_property("editable", False)
	else:
		renderer.set_property("editable", True)

def configValueEdited(renderer, path, newText):
	model = widgets.get_object("configView").get_model()
	treeIter = model.get_iter(path)

	catIter = model.iter_parent(treeIter)

	if not catIter:
		return

	model.set(treeIter, 1, newText)

	category = model.get(catIter, 0)[0]
	option = model[path][0]
	value = model[path][1]

	config.set(category, option, value)

def setup():
	""" called initially """
	global widgets

	widgets = gui.builder.load_dialog("advancedPreferences")

	configView = widgets.get_object("configView")

	widgets.get_object("value_renderer").connect("edited",
			configValueEdited)
	widgets.get_object("value_column").set_cell_data_func(
			widgets.get_object("value_renderer"),
			renderValue)
	widgets.get_object("option_column").set_cell_data_func(
			widgets.get_object("option_renderer"),
			renderOption)


def run():
	dialog = widgets.get_object("advancedPreferences")

	fillConfigView()

	dialog.run()
	dialog.destroy()



# coding: UTF-8
"""
Copyright (c) 2008 Marian Tietz
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
"""

from re import compile

from gettext import gettext as _

import gtk
import gobject

import logging

from .. import com
from .. import signals

from ..gui import builder
from ..gui import mgmt
from ..helper.markup import markup_escape

PULSE_DELAY=100

running = False # dialog is running
cache = []  # /list cache

class ChannelListBuilder(builder.Builder):

	def __init__(self, server_name):
		super(ChannelListBuilder, self).__init__()

		self.load_dialog("channelList")

		self.server_name = server_name
		self.pulse_timer = None

		self.connect_signals(self)

		self.reset_progress_bar()

	def reset_progress_bar(self):
		self.get_object("progressBar").set_fraction(0)

	def start_pulsing(self):
		def pulse_it():
			self.get_object("progressBar").pulse()
			return True

		self.pulse_timer = gobject.timeout_add(PULSE_DELAY, pulse_it)

	def stop_pulsing(self):
		if not self.pulse_timer:
			return
		gobject.source_remove(self.pulse_timer)
		self.reset_progress_bar()

	def start_list(self):
		pattern = None

		try:
			pattern = compile(
				self.get_object("regexpEntry").get_text())

		except Exception as e:
			mgmt.show_inline_message(
				_("Channel list search error."),
				_("You've got a syntax error in your search string. "
					"The error is: %s\n"
					"<b>Tip:</b> You should not use special characters "
					"like '*' or '.' in your search string if you don't "
					"know about regular expressions." % (e)),
				dtype="error")


		self.get_object("listStore").clear()

		self.get_object("listButton").set_sensitive(False)
		self.get_object("stopListButton").set_sensitive(True)

		self.start_pulsing()

		if cache:
			# use cached values
			for (server, channel, user, topic) in cache:
				self.list_handler(0, server, channel, user, topic, pattern)

		else:
			signals.connect_signal("list", self.list_handler, pattern)

			try:
				com.sushi.list(self.server_name, "")

			except Exception as e:
				logging.error("Error in getting list: %s" % (e))
				self.stop_list()

	def stop_list(self):
		signals.disconnect_signal("list", self.list_handler)

		self.get_object("listButton").set_sensitive(True)
		self.get_object("stopListButton").set_sensitive(False)

		self.stop_pulsing()

	def list_handler(self, time, server, channel, user, topic, pattern):
		""" receives the data from maki.
			add server/user/topic to listStore
		"""

		if time > 0:
			# no manual call
			cache.append((server,channel,user,topic))

		if user < 0:
			# EOL, this is not reached if we use
			# manual call.
			self.stop_list()
			return

		store = self.get_object("listStore")

		if (not pattern
		or (pattern and (pattern.search(channel)
					or pattern.search(topic)))):

			store.append(row=(markup_escape(channel), int(user),
							  markup_escape(topic)))


	def dialog_response(self, dialog, id):
		global running

		self.stop_list()
		running = False
		dialog.destroy()

	def find_button_clicked(self, button):
		self.start_list()

	def stop_button_clicked(self, button):
		# prevent incompleteness of cache
		global cache
		cache = []

		self.stop_list()

	def regexp_entry_activate(self, entry):
		self.start_list()

	def listView_row_activated(self, view, path, column):
		channel = self.get_object("listStore")[path][0]

		com.sushi.join(self.server_name, channel,
				com.sushi.server_get(self.server_name, channel, "key"))




def run(server):
	""" Show the dialog until close was hit. """

	global cache
	global running

	if running:
		return

	running = True

	# clear the cache at the beginning
	cache = []

	builder = ChannelListBuilder(server)
	d = builder.get_object("channelList")

	main_window = mgmt.widgets.get_object("main_window")
	d.set_transient_for(main_window)

	d.show()


def setup(): pass



# coding: UTF-8
"""
Copyright (c) 2009 Marian Tietz
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
"""

import os
import sys
import time
from typecheck import types

from xdg.BaseDirectory import xdg_data_home

class Logger (object):

	_path = ""
	_file = None

	def __init__ (self, filename, open=True):
		self._set_path (filename)

		if open: self.open ()

	def open (self):
		""" open the log file and write start message to it """
		if not self._check_path ():
			return False

		self._file = file (self._path, "a+")

		timestamp = time.strftime (
			">>> Logging started at %H:%M on %d. %b %Y.")

		self.write (timestamp+"\n")

		return True

	def get_path(self):
		return self._path

	def silent_close (self):
		if self._file or self._file.closed:
			self._file.close()

	def close (self):
		""" write stop text to file and close it """

		if self._file:
			self.write ("%s\n" % (time.strftime (
				">>> Logging stopped at %H:%M on %d. %b. %Y.")))
			self.silent_close ()

	@types (text=str)
	def write (self, text):
		""" write timestamp plus text to log file """

		if not self._file or self._file.closed or not text:
			return

		try:
			self._file.write (text)
		except ValueError:
			# despite of is-closed-check, file is closed
			return

	@types (lines=int)
	def read (self, lines):
		if not self._file or self._file.closed:
			return None

		# TODO: implement this
		return ()

	def _set_path (self, filename):
		self._path = os.path.join (
			xdg_data_home,
			"sushi",
			filename)

	def _check_path (self):
		""" check for every dir on the way
			to the log file and create it
			if not found.
		"""
		path, file = os.path.split(self._path)

		if not os.path.exists (path):
			# create the directories
			try:
				os.makedirs (path)
			except os.error, e:
				print "Error while creating neccessary directories: %s"\
					% (e)
				return False

		if not os.path.exists(os.path.join(path, file)):
			try:
				f = open (self._path, "w")
			except BaseException,e:
				print "Error while creating error log file: %s" % (e)
				return False
			else:
				f.close()

			return True
		else:
			return True
		return False

	def _handle_main_quit (self):
		self.close()
